<?php
c_include('forms/formField');

/**
 * Form Field class for the Mideas Framework.
 *
 * @package		Mideas.Framework
 * @subpackage	Form
 * @since		1.6
 */
class FormFieldMplayuitoolbar extends FormField
{
	/**
	 * The form field type.
	 *
	 * @var		string
	 * @since	1.6
	 */
	protected $type = 'mplayuitoolbar';

	/**
	 * Method to get the field input markup.
	 *
	 * @return	string	The field input markup.
	 * @since	1.6
	 */
	public function getInput($valor=null)
	{
		// Initialize variables.
		$html = array();
		$attr = '';

		// Initialize some field attributes.
		$attr .= $this->element['class'] ? ' class="'.(string) $this->element['class'].'"' : '';
				
		e_include('ext_mplay/class/mplayUi');
		if (!class_exists("MplayUI"))
			return false;
			
		$tools = MPlayUI::getAllTools();
		
		$all_tools_options = '<option value="">'.__('Ninguna herramienta').'</option>';
		foreach ($tools as $typetool=>$toolset){
			if ($typetool == 'core'){
				foreach ($toolset as $tool){
					$all_tools_options .= '<option value="'.$tool.'">'.ucfirst(str_replace("-", " ", $tool)).'</option>';
				}
			} elseif ($typetool == 'modulos'){
				foreach ($toolset as $modulo=>$mtools){
					$check_mod = MplayModulo::getInstance($modulo);
					
					if (!$check_mod instanceof MplayModulo)
						continue;
					
					$all_tools_options .= '<optgroup label="'.$check_mod->getNombre().'">';	
					foreach ($mtools as $tool){
						$tool_check = "mod_" . $check_mod->getId() . ':';
						
						if (substr($tool, 0, strlen($tool_check)) == $tool_check){
							$tool_name = substr($tool, strlen($tool_check));
							$all_tools_options .= '<option value="'.$tool.'">'.ucfirst(str_replace("-", " ", $tool_name)).'</option>';
						}
					}
					$all_tools_options .= '</optgroup>';
				}
			}
		}
		
		$output = '';
		
		$output .= '<div class="row_mplay_toolbar_'.$this->id.' panel panel-default">
		<ul class="list-group">
		';
		
		foreach ($valor as $val_enc){
			parse_str($val_enc, $val);
			
			if (!is_array($val) || count($val) != 3)
				continue;
				
			$output .= '<li class="list-group-item"><i class="'.$val['icono'].'"></i> '.$val['titulo'].' <small>('.$val['tipo'].')</small><a href="javascript:;" class="pull-right btn btn-xs btn-danger" onClick="$(this).parent().remove();"><i class="fa fa-times"></i></a><input type="hidden" name="'.$this->name.'[]" value="'.$val_enc.'" /></li>';
		}
		
		$output .= '</ul></div>
		<a href="#mdl_add_mplay_toolbar_'.$this->id.'" data-toggle="modal" class="btn btn-primary"><i class="fa fa-plus"></i> '.__('Nueva herramienta').'</a>
		
		<div class="modal fade" tabindex="-1" role="dialog" id="mdl_add_mplay_toolbar_'.$this->id.'">
		  <div class="modal-dialog">
			<div class="modal-content">
			  <div class="modal-header">
				<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
				<h4 class="modal-title">'.__('Añadir herramienta').'</h4>
			  </div>
			  <div class="modal-body">
				<div class="row">
					<div class="form-group col-sm-4"><label>'.__('Título').'</label><input type="text" class="form-control" name="titulo" id="inp_titulo_mplay_element_'.$this->id.'" value="" /></div>
					<div class="form-group col-sm-4"><label>'.__('Icono').'</label><input type="text" class="form-control" name="icono" id="inp_icono_mplay_element_'.$this->id.'" value="" /></div>
					<div class="form-group col-sm-4"><label>'.__('Tipo').'</label><select class="form-control" name="tipo" id="inp_tipo_mplay_element_'.$this->id.'">'.$all_tools_options.'</select></div>
				</div>
			  </div>
			  <div class="modal-footer">
				<button type="button" class="btn btn-default" data-dismiss="modal">'.__('Cerrar').'</button>
				<button id="add_mplay_toolbar_'.$this->id.'" type="button" class="btn btn-primary">'.__('Añadir herramienta').'</button>
			  </div>
			</div><!-- /.modal-content -->
		  </div><!-- /.modal-dialog -->
		</div><!-- /.modal -->
		';
		
		$GLOBALS['AppDoc']->AddScriptDeclaration('
		$(document).ready(function(){
			$(".row_mplay_toolbar_'.$this->id.' > ul.list-group").sortable();
			
			$("button#add_mplay_toolbar_'.$this->id.'").on("click", function(){
				var row = $(".row_mplay_toolbar_'.$this->id.'");
				
				var titulo = $("input#inp_titulo_mplay_element_'.$this->id.'");
				var icono = $("input#inp_icono_mplay_element_'.$this->id.'");
				var tipo = $("select#inp_tipo_mplay_element_'.$this->id.'");
				
				if (titulo.val() == ""){
					bootbox.alert("'.__('Debes escribir un título a la herramienta').'");
					return false;
				}
				
				if (icono.val() == ""){
					bootbox.alert("'.__('Debes elegir un icono de la herramienta').'");
					return false;
				}
				
				if (tipo.val() == ""){
					bootbox.alert("'.__('Debes elegir un tipo de herramienta').'");
					return false;
				}
				
				var serialized = $.param({ titulo: titulo.val(), icono: icono.val(), tipo: tipo.val() });
				
				row.find("ul.list-group").append("<li class=\"list-group-item\"><i class=\""+icono.val()+"\"></i> "+titulo.val()+" <small>("+tipo.val()+")</small><a href=\"javascript:;\" class=\"pull-right btn btn-xs btn-danger\" onClick=\"$(this).parent().remove();\"><i class=\"fa fa-times\"></i></a><input type=\"hidden\" name=\"'.$this->name.'[]\" value=\""+serialized+"\" /></li>");
				
				row.find("ul.list-group").sortable();
				
				$("#mdl_add_mplay_toolbar_'.$this->id.'").modal("hide");
				
				return false;
			});
		});
		');
		

		return $output;
	}
}
