<?php
defined('CONF_URL_APP') or die;

class Juego extends ItemModel
{
	protected $dbt_const = "DBT_JUEGOS";
	protected $dbt = "mplay_juegos";
	
	protected $ext_folder = "ext_mplay";
	protected $ext_vista_categoria = "categoria_mplay";
	protected $ext_vista_item = "juego";
	
	protected $ext_item_class = "Juego";
	protected $ext_categoria_class = "MplayCategoria";
	protected $ext_item_campoextra_class = "";
	
	protected $all_vars = array("id","titulo","alias","contenido","texto_corto","fecha","fecha_publicacion","estado","imagen_destacada","categorias","creador","creador_alias","propietario","params","tags","idioma","lugar","direccion","codigo_postal","localidad","provincia","pais","latitud","longitud","location","acceso","oficio","id_original", "radio", "visibilidad", "tipo");
	
	protected $idx_vars = array("id");
	protected $update_vars = array("titulo","alias","contenido","texto_corto","fecha_publicacion","estado","imagen_destacada","categorias","creador","creador_alias","propietario","params","tags","idioma","lugar","direccion","codigo_postal","localidad","provincia","pais","latitud","longitud","location","acceso","oficio","id_original", "radio", "visibilidad", "tipo");
	
	protected $campos_multilang = array("titulo","alias","contenido","texto_corto","lugar");
	protected $campos_array = array("params","categorias");
	
	protected $plantilla_info_html = '
                            <a href="##URL##" class="item-map relative" style="display: block;" data-lat="##LATITUD##" data-lng="##LONGITUD##" data-icono="##CATEGORIAICONO##" data-color="green" data-caption="<p><a href=\'##URL##\'>##TITULO##</a></p>"><div class="panel panel-default" style="margin-top: 10px;"><div class="panel-body">
							<div class="col-xs-12 col-sm-2 col-md-2 col-lg-2" style="padding:0;"> <img class="thumbnail" style="margin-bottom:0;" src="##IMAGEN##" width="100%"> </div>
                            <div class="col-xs-12 col-sm-10 col-md-10 col-lg-10">
            <h4><b>##TITULO##</b><br><small>##LOCALIZACION##</small></h4>
            <small>##CATEGORIAS##</small>
                            </div>
                            </div></div></a>';
	
	function __construct(){
		parent::__construct();
	}
	
	function includes(){
		c_include('loc/region','loc/provincia','loc/localidad','loc/pais');
		e_include('ext_mplay/class/mplayCategoria','ext_mplay/class/misionJuegoMplay');
	}
	
	function checkAcceso(){
		$parent_acceso = parent::checkAcceso();
		
		if ($parent_acceso === true){
			if (Usuario::es_superadmin()){
				return true;
			}
			switch ($this->visibilidad){
				case "1":
					return true;
					break;
				case "0":
					if (!Usuario::isLoged() || $this->propietario != '-'.Usuario::getLogedId()){
						return false;
					} else {
						return true;
					}
					break;
				case "2":
					if (!Usuario::isLoged()){
						return false;
					}
					$propietario_juego = $this->getpropietario(true,true);
					
					if (!is_object($propietario_juego)){
						return true;
					}
					
					if (Usuario::getLogedId() == $propietario_juego->id)
						return true; 
					
					$seguidores_propietario = $propietario_juego->get_seguidores();
					
					if (!is_array($seguidores_propietario) || sizeof($seguidores_propietario) == 0)
						return false;
						
					foreach ($seguidores_propietario as $sp){
						if ($sp->id == Usuario::getLogedId()){
							return true;
						}
					}
					
					return false;
					break;
			}
			return true;
		} else {
			return $parent_acceso;
		}
	}
	
	function getMisiones($solo_activos=true){
		if (empty($this->id))
			return false;
			
		$extra_sql = '';
		if ($solo_activos == true)
			$extra_sql .= " AND estado='1'";
			
		$misiones = new MisionJuegoMplay();
		$misiones = $misiones->Listar("juego='".$this->id."'".$extra_sql, "orden", "ASC");
		
		return $misiones;
	}
	
	function iniciado(){
		if (!Usuario::isLoged())
			return false;
			
		$usr_obj = Usuario::getLoged();
		$usr_id = $usr_obj->id;
		
		e_include('ext_mplay/class/helpers/mplayUsuarioHelper');
		
		if (!class_exists('MplayUsuarioHelper')){
			return false;
		}
		
		$juegos_activos_usr = MplayUsuarioHelper::getJuegosActivos($usr_obj);
		
		if (in_array($this->id, $juegos_activos_usr)){
			return true;
		} else {
			return false;
		}
	}
	
	function completado(){
		if (!Usuario::isLoged())
			return false;
			
		$usr_obj = Usuario::getLoged();
		$usr_id = $usr_obj->id;
		
		e_include('ext_mplay/class/helpers/mplayUsuarioHelper');
		
		if (!class_exists('MplayUsuarioHelper')){
			return false;
		}
		
		$juegos_finalizados_usr = MplayUsuarioHelper::getJuegosFinalizados($usr_obj);
		
		if (in_array($this->id, $juegos_finalizados_usr)){
			return true;
		} else {
			return false;
		}
	}
	
	function completar(){
		if ($this->completado())
			return false;
			
		if (!Usuario::isLoged())
			return false;
			
		$usr_obj = Usuario::getLoged();
		$usr_id = $usr_obj->id;
			
		$params_juego = Helpers::toArray(json_decode($this->params));
		$propiedades_juego = $params_juego['propiedades'];
		
		e_include('ext_mplay/class/mplayRecompensaTipo');
		
		if (!empty($params_juego['recompensa1']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa1']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_aplicada = $recompensa_tipo->aplicar($params_juego['recompensa1']['valor']);
			}
		}
		
		if (!empty($params_juego['recompensa2']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa2']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_aplicada = $recompensa_tipo->aplicar($params_juego['recompensa2']['valor']);
			}
		}
		
		if (!empty($params_juego['recompensa3']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa3']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_aplicada = $recompensa_tipo->aplicar($params_juego['recompensa3']['valor']);
			}
		}
		
		if ($params_juego['generar_actividad_recompensa'] == '1' && !empty($params_juego['texto_actividad_recompensa'][SistemaIdioma::getIdiomaSesion()])){
			$img_recompensa = '';
			if (!empty($params_juego['icono_recompensa'])){
				$img_recompensa = Helpers::generateImageLink($params_juego['icono_recompensa'], 48, 48);
				$img_recompensa = '<img src="'.$img_recompensa.'" width="48" height="48" /> ';
			}
			$txt_actividad = $img_recompensa . $params_juego['texto_actividad_recompensa'][SistemaIdioma::getIdiomaSesion()];
			$usr_obj->nuevaActividad($txt_actividad, 'new', 0, array("tipo" => 'ext_mplay.recompensa_juego', "id_tipo" => $this->id, "id_recompensa"=>$params_juego['recompensas']['tipo']));
		}
		
		return true;
	}
	
	function printRecompensas($cls=''){
		$params_juego = Helpers::toArray(json_decode($this->params));
		$propiedades_juego = $params_juego['propiedades'];
		
		e_include('ext_mplay/class/mplayRecompensaTipo');
		
		$recompensa_html = '';
		
		if (!empty($params_juego['recompensa1']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa1']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_html .= $recompensa_tipo->printHtml($params_juego['recompensa1']['valor'], $cls);
			}
		}
		
		if (!empty($params_juego['recompensa2']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa2']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_html .= $recompensa_tipo->printHtml($params_juego['recompensa2']['valor'], $cls);
			}
		}
		
		if (!empty($params_juego['recompensa3']['tipo'])){
			$recompensa_tipo = MplayRecompensaTipo::getInstance($params_juego['recompensa3']['tipo']);
			
			if ($recompensa_tipo instanceof MplayRecompensaTipo){
				$recompensa_html .= $recompensa_tipo->printHtml($params_juego['recompensa3']['valor'], $cls);
			}
		}
		
		return $recompensa_html;
	}
	
	public static function getTypes()
	{
		e_include('ext_mplay/class/mplayJuegoTipo');
		
		$dir = dirname(__FILE__) . '/juegos';
		$handlers = Helpers::dir_list($dir,'','.php',false);
		$handlers = $handlers[dirname(__FILE__) . '/juegos'];
		
		$names = array();
		foreach ($handlers as $handler)
		{
			$name = substr($handler, 0, strrpos($handler, '.'));
			$class = 'MplayJuegoTipo' . ucfirst($name);

			if (!class_exists($class))
			{
				include_once dirname(__FILE__) . '/juegos/' . $name . '.php';
			}

			if (call_user_func_array(array(trim($class), 'test'), array()))
			{
				$names[] = $name;
			}
		}

		return $names;
	}
}
?>