<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

class MplayMisionTipoConseguir_habilidades extends MplayMisionTipo
{
	protected $name = 'Conseguir habilidades';
	protected $description = 'Misión que consiste en conseguir las habilidades propuestas';
	
	protected $acciones_mapa = array('comenzar','ir');
	
	public function __construct($options=array()){
		$this->propiedades = array(
				"habilidades"	=>	array(
								"nombre"	=>	__("Habilidades"), 
								"tipo"		=>	"string", 
								"input"		=>	"mplayhabilidadeslist", 
								"icono"		=>	"IT", 
								"descripcion"	=>	__("Habilidades de la misión"),
								"params"	=>	array("multiple"=>"true"),
								"visible"	=>	false
							),
				"condicion"	=>	array(
								"nombre"	=>	__("Condición"), 
								"tipo"		=>	"string",
								"input"		=>	"list",
								"icono"		=>	"CD",
								"descripcion"	=>	__("Condición de la misión"),
								"visible"	=>	false,
								"params"	=>	array("opciones" => array("todos"=>__('Conseguir todas las habilidades'), "alguno"=>__('Conseguir al menos una habilidad')))
							),
				"tiempo"	=>	array(
								"nombre"	=>	__("Tiempo límite (en minutos)"), 
								"tipo"		=>	"string", 
								"input"		=>	"number", 
								"icono"		=>	"TM", 
								"descripcion"	=>	__("Tiempo límite en minutos para completar la misión"), 
								"params"	=>	array("class"=>"form-control")
							),
				"jsfunc"	=>	array(
								"nombre"	=>	__("Función JS de la misión"), 
								"tipo"		=>	"string", 
								"input"		=>	"hidden", 
								"icono"		=>	"JS", 
								"descripcion"	=>	__("Función JS de la misión"),
								"valor"		=>	"MPlayConseguir_habilidadesMission",
								"value"		=>	"MPlayConseguir_habilidadesMission",
								"params"	=>	array("valor"=>"MPlayConseguir_habilidadesMission", "value"=>"MPlayConseguir_habilidadesMission"),
								"visible"	=>	false
							)
						);
						
		parent::__construct($options);
	}
	
	function addScriptMission(){
		return $GLOBALS['AppDoc']->addScriptDeclaration('
		var trMPlayConseguir_habilidadesMission = [];
		
		function MPlayConseguir_habilidadesMission(datos){
			if (MPlayJuegos.habilidades_mision == undefined){
				MPlayJuegos.habilidades_mision = [];
			}
			MPlayJuegos.habilidades_mision[datos.id_mision] = datos.propiedades_mision.habilidades;
			var num_habilidades = MPlayJuegos.habilidades_mision[datos.id_mision].length;
			
			if (MPlayJuegos.timeoutMPlayConseguir_habilidadesMission == undefined){
				MPlayJuegos.timeoutMPlayConseguir_habilidadesMission = [];
			}
						
			if (num_habilidades > 0){
				var jq_info = "#mplay-ui-info-mission-"+datos.id_mision;
				$(jq_info).removeClass("hide").html("<i class=\'fa fa-flask\'></i> 0/"+num_habilidades.toString());
			}
			
			if (datos.propiedades_mision.tiempo != undefined && datos.propiedades_mision.tiempo != ""){
				trMPlayConseguir_habilidadesMission[datos.id_mision] = parseInt(datos.propiedades_mision.tiempo * 60);
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				$(jq_timer).removeClass("hide").html("<i class=\'fa fa-clock-o\'></i> "+MideasFunciones.formatoHora(trMPlayConseguir_habilidadesMission[datos.id_mision]));
			}
						
			MPlayJuegos.timeoutMPlayConseguir_habilidadesMission[datos.id_mision] = setInterval(timerMPlayConseguir_habilidadesMission, 1000, datos);
		}
		function timerMPlayConseguir_habilidadesMission(datos){
			var num_habilidades = MPlayJuegos.habilidades_mision[datos.id_mision].length;
			
			if (trMPlayConseguir_habilidadesMission[datos.id_mision] != null){
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				if (!$(jq_timer).is(":visible")){
					$(jq_timer).show();
				}
				$(jq_timer).html("<i class=\'fa fa-clock-o\'></i> "+MideasFunciones.formatoHora(trMPlayConseguir_habilidadesMission[datos.id_mision]));
				if (trMPlayConseguir_habilidadesMission[datos.id_mision] <= 0){
					MPlayJuegos.mission_expired(datos);
					clearInterval(MPlayJuegos.timeoutMPlayConseguir_habilidadesMission[datos.id_mision]);
					$(jq_timer).hide();
				} else {
					trMPlayConseguir_habilidadesMission[datos.id_mision] = trMPlayConseguir_habilidadesMission[datos.id_mision] - 1;
				}
			}
			
			var num_conseguidos = 0;
			var hab_array = $.makeArray(MPlayUsuario.habilidades);
			
			if (hab_array != undefined && hab_array.length > 0){
				$.each(hab_array, function(idxi, elei){
					if ($.inArray(elei.id.toString(), datos.propiedades_mision.habilidades) != -1){
						num_conseguidos = num_conseguidos + 1;
					}
				});
			}
			
			var jq_info = "#mplay-ui-info-mission-"+datos.id_mision;
			$(jq_info).removeClass("hide").html("<i class=\'fa fa-flask\'></i> "+num_conseguidos.toString()+"/"+num_habilidades.toString());
						
			if ((datos.propiedades_mision.condicion == "alguno" && num_conseguidos > 0) || num_conseguidos == num_habilidades){
				var jq_info = "#mplay-ui-info-mission-"+datos.id_mision;
				$(jq_info).addClass("hide").html("");
				if (MPlayJuegos.habilidades_mision[datos.id_mision] != undefined){
					delete MPlayJuegos.habilidades_mision[datos.id_mision];
				}
				MPlayJuegos.mission_completed(datos);
				clearInterval(MPlayJuegos.timeoutMPlayConseguir_habilidadesMission[datos.id_mision]);
			}
		}
		function MPlayConseguir_habilidadesMissionDel(idm){
			var jq_info = "#mplay-ui-info-mission-"+idm;
			$(jq_info).addClass("hide").html("");
			clearInterval(MPlayJuegos.timeoutMPlayConseguir_habilidadesMission[idm]);
		}
		');
	}
}