<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

class MplayMisionTipoDinamica extends MplayMisionTipo
{
	protected $name = 'Dinámica';
	protected $description = 'Misión que establece la posición de la zona de activación respecto a la posición actual del jugador';
	
	protected $acciones_mapa = array('comenzar','ir');
	
	public function __construct($options=array()){
		$this->propiedades = array(
				"angulo"	=>	array(
								"nombre"	=>	__("Ángulo de la zona de activación"), 
								"tipo"		=>	"int", 
								"input"		=>	"number", 
								"icono"		=>	"DS", 
								"descripcion"	=>	__("Ángulo de la zona de activación de la misión respecto a la posición del jugador"),
								"params"	=>	array("class"=>"form-control"),
								"visible"	=>	false
							), 
				"distancia"	=>	array(
								"nombre"	=>	__("Distancia de la zona de activación"), 
								"tipo"		=>	"int", 
								"input"		=>	"number", 
								"icono"		=>	"DS", 
								"descripcion"	=>	__("Distancia de la zona de activación de la misión respecto a la posición del jugador"),
								"params"	=>	array("class"=>"form-control"),
								"visible"	=>	false
							), 
				"radio"	=>	array(
								"nombre"	=>	__("Radio de la zona de activación"), 
								"tipo"		=>	"int", 
								"input"		=>	"number", 
								"icono"		=>	"RA", 
								"descripcion"	=>	__("Radio de la zona de activación de la misión"),
								"params"	=>	array("class"=>"form-control"),
								"visible"	=>	false
							), 
				"tiempo"	=>	array(
								"nombre"	=>	__("Tiempo límite (en minutos)"), 
								"tipo"		=>	"string", 
								"input"		=>	"number", 
								"icono"		=>	"TM", 
								"descripcion"	=>	__("Tiempo límite en minutos para completar la misión"), 
								"params"	=>	array("class"=>"form-control")
							),
				"jsfunc"	=>	array(
								"nombre"	=>	__("Función JS de la misión"), 
								"tipo"		=>	"string", 
								"input"		=>	"hidden", 
								"icono"		=>	"JS", 
								"descripcion"	=>	__("Función JS de la misión"),
								"valor"		=>	"MPlayDinamicaMission",
								"visible"	=>	false
							)
						);
						
		parent::__construct($options);
	}
	
	function addScriptMission(){
		return $GLOBALS['AppDoc']->addScriptDeclaration('
		var trMPlayDinamicaMission = [];
		
		function MPlayDinamicaMission(datos){
			if (MPlayMap.marker != undefined){
				var marker = MPlayMap.marker;
				var mk = marker.getLatLng();
			} else {
				var lat = parseFloat(MPlay.usuario_activo.actividad.latitud);
				var lng = parseFloat(MPlay.usuario_activo.actividad.longitud);
				var mk = L.latLng(lat,lng);
			}
			
			var m = MPlayMap.destinationPoint(mk, datos.propiedades_mision.angulo, datos.propiedades_mision.distancia);
			if (MPlayJuegos.misiones == undefined){
				MPlayJuegos.misiones = [];
			}
			var cur_elem = $("#mplay-ui-btn-mission-"+datos.juego_mision);
			cur_elem.data("lat", m.lat).data("lng", m.lng);
			MPlayJuegos.misiones[datos.id_mision] = L.circle(m, datos.propiedades_mision.radio, { color: "blue", fillColor: "#03f", fillOpacity: 0.5, clickable: false }).addTo(MPlayMap.map);
			
			if (MPlayJuegos.timeoutMPlayDinamicaMission == undefined){
				MPlayJuegos.timeoutMPlayDinamicaMission = [];
			}
			
			if (datos.propiedades_mision.tiempo != undefined && datos.propiedades_mision.tiempo != ""){
				trMPlayDinamicaMission[datos.id_mision] = parseInt(datos.propiedades_mision.tiempo * 60);
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				$(jq_timer).removeClass("hide").html(MideasFunciones.formatoHora(trMPlayDinamicaMission[datos.id_mision]));
			}
						
			MPlayJuegos.timeoutMPlayDinamicaMission[datos.id_mision] = setInterval(timerMPlayDinamicaMission, 1000, datos);
		}
		function timerMPlayDinamicaMission(datos){
			var areaMap = MPlayJuegos.misiones[datos.id_mision];
			var bounds = areaMap.getBounds();
			var mk = MPlayMap.marker;
			
			if (trMPlayDinamicaMission[datos.id_mision] != null){
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				if (!$(jq_timer).is(":visible")){
					$(jq_timer).show();
				}
				$(jq_timer).html(MideasFunciones.formatoHora(trMPlayDinamicaMission[datos.id_mision]));
				if (trMPlayDinamicaMission[datos.id_mision] <= 0){
					MPlayMap.map.removeLayer(MPlayJuegos.misiones[datos.id_mision]);
					MPlayJuegos.mission_expired(datos);
					clearInterval(MPlayJuegos.timeoutMPlayDinamicaMission[datos.id_mision]);
					$(jq_timer).hide();
				} else {
					trMPlayDinamicaMission[datos.id_mision] = trMPlayDinamicaMission[datos.id_mision] - 1;
				}
			}
			if (bounds.contains(mk.getLatLng())){
				MPlayMap.map.removeLayer(MPlayJuegos.misiones[datos.id_mision]);
				if (trMPlayDinamicaMission[datos.id_mision] != null){
					var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
					$(jq_timer).hide().html("");
				}
				MPlayJuegos.mission_completed(datos);
				clearInterval(MPlayJuegos.timeoutMPlayDinamicaMission[datos.id_mision]);
			}
		}
		function MPlayDinamicaMissionDel(idm){
			MPlayMap.map.removeLayer(MPlayJuegos.misiones[idm]);
			clearInterval(MPlayJuegos.timeoutMPlayDinamicaMission[idm]);
		}
		');
	}
}