<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  UI
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

class MplayModuloInventario extends MplayModulo
{
	protected $id = 'inventario';
	protected $nombre = 'Inventario';
	protected $icono = 'fa fa-suitcase';
	protected $descripcion = 'Módulo de mPlay para habilitar la gestión de un inventario de items';
	
	public function __construct($options = array()){
		$this->root_path = dirname(__FILE__);
		
		parent::__construct($options);
	}
	
	function onNearestItems($p=array()){
		$lat = $p['lat'];
		$lng = $p['lng'];
		$radio = $p['radio'];
		
		if (empty($lat) || empty($lng))
			return false;
		
		$usr_id = Usuario::isLoged() ? Usuario::getLogedId() : 0;
		$usr_obj = Usuario::isLoged() ? Usuario::getLoged() : false;

		if (empty($radio)){
			if ($usr_obj){
				$radio = (float) MplayUsuarioHelper::getRangoVision($usr_obj); // en kilometros
			} else {
				$radio = $GLOBALS['opciones']['ext_mplay']['usuario']['rango_vision'];
			}
		}
			
		if ($usr_id > 0){
			$usr_loged = Usuario::getLoged();
			$usr_loged_seguidores = new ItemsSeguidores();
			$usr_loged_seguidores_c = $usr_loged_seguidores->ListarCount("item_id='ext_usuarios.".$usr_id."'");
			
			$usrs_where = '';
			if ($usr_loged_seguidores_c > 0){
				$usr_loged_seguidores = $usr_loged_seguidores->Listar("item_id='ext_usuarios.".$usr_id."'");
				
				$usrs_where = " OR (visibilidad = '2' AND creador IN (";
				
				$ii = 0;
				foreach ($usr_loged_seguidores as $uls){
					$ii ++;
					$usrs_where .= "'-".$uls->usuario_id."'";
					
					if ($ii < $usr_loged_seguidores_c){
						$usrs_where .= ', ';
					}
				}
				
				$usrs_where .= "))";
			}
			
			$where_loc = 'propietario = "0" AND estado = "1" AND bot = "0" AND sitio = "0" AND ((visibilidad = "1") OR (visibilidad = "0" AND creador = "-'.$usr_id.'")'.$usrs_where.')';
		} else {
			$where_loc = 'propietario = "0" AND estado = "1" AND (visibilidad = "1") AND sitio = "0" AND bot = "0"';
		}
		
		$mua = new MplayItem();
		
		$mua_chk_location = $mua->ListarCountPorLocalizacion($lat, $lng, $radio, 0, 0, array($where_loc));
		
		if ($mua_chk_location > 0){
			
			$mua_location = $mua->ListarPorLocalizacion($lat, $lng, $radio, 0, 0, array($where_loc));
			
			if ($usr_obj){
				$radio_interaccion = (float) MplayUsuarioHelper::getRangoInteraccion($usr_obj); // en kilometros
			} else {
				$radio_interaccion = (float) $GLOBALS['opciones']['ext_mplay']['usuario']['rango_interaccion'];
			}
			
			$array_json = array();
			
			foreach ($mua_location as $ml){
				$distancia_ml = (float) $ml->get_distancia(false);
				$tipo_item_obj = MplayItemTipo::getInstance($ml->tipo, Helpers::toArray(json_decode($ml->propiedades)));
				
				$params_ml = Helpers::toArray(json_decode($ml->params));
				
				$onuse = $tipo_item_obj->onUse();
				
				if ($tipo_item_obj->onShow()){
					if ($distancia_ml < $radio_interaccion){
						$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".SistemaIdioma::Imprimir($ml->nombre)."<br><small>".$ml->get_distancia(true)."</small></h4><div class='mplay-popup-description'>".SistemaIdioma::Imprimir($ml->descripcion)."</div><p>".$tipo_item_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><hr class='light'><div class='mplay-popup-options'><div class='row'>".$tipo_item_obj->printOpcionesBtns('mapa', $ml)."</div></div></div>";
					} else {
						$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".SistemaIdioma::Imprimir($ml->nombre)."<br><small>".$ml->get_distancia(true)."</small></h4><p>".$tipo_item_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><div class='mplay-popup-description'>".SistemaIdioma::Imprimir($ml->descripcion)."</div><div class='mplay-popup-alert alert alert-info'>".__("Estás demasiado lejos para interactuar con el objeto.")."</div><p><small>".sprintf(__('Estás a %s del item y necesitas estar a %s como mínimo para poder interactuar con él.'), $ml->get_distancia(true), number_format($radio_interaccion * 1000, 0).' '.__('metros'))."</small></p><div class='clearfix'></div><hr class='light'><div class='mplay-popup-options'>
  <div class='row'>
    <div class='mplay-opt col-md-12'><a href='javascript:;' class='mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e' data-lat='".(float) $ml->latitud."' data-lng='".(float) $ml->longitud."' title='".__('Ir')."'><i class='fa fa-crosshairs'></i> ".__('Ir')."</a></div>
  </div>
</div></div>";
					}
					
					$ancho_icono = $GLOBALS['opciones']['ext_mplay']['ui']['ancho_icono_items'];
					$alto_icono = $GLOBALS['opciones']['ext_mplay']['ui']['alto_icono_items'];
					if (!empty($params_ml['ancho_icono']) && !empty($params_ml['alto_icono'])){
						$ancho_icono = $params_ml['ancho_icono'];
						$alto_icono = $params_ml['alto_icono'];
					}
					if (empty($ancho_icono)){
						$ancho_icono = 64;
					}
					if (empty($alto_icono)){
						$alto_icono = 64;
					}
					$icono = $ml->geticono($ancho_icono, $alto_icono);
					if (empty($icono) && !empty($GLOBALS['opciones']['ext_mplay']['ui']['icono_items'])){
						$icono = Helpers::generateImageLink($GLOBALS['opciones']['ext_mplay']['ui']['icono_items'], $ancho_icono, $alto_icono);
					}
					
					$tmp_array = array('type'=>'Feature', 'geometry'=>array('type'=>'Point', 'coordinates'=>array((float) $ml->latitud, (float) $ml->longitud)), 'properties'=>array('nombre'=>SistemaIdioma::Imprimir($ml->nombre), 'imagen'=>$icono, 'ancho_icono'=>$ancho_icono, 'alto_icono'=>$alto_icono, 'distancia'=>$ml->get_distancia(true), 'content'=>$popup_content, 'tipo'=>'item', 'propiedades_html'=>$tipo_item_obj->printPropiedadesHtml(' class="col-xs-12"'), 'propiedades'=>$tipo_item_obj->propiedades), 'id'=>'item_'.$ml->id);
					
					$array_json[] = $tmp_array;
				}
			}
			
			return $array_json;
		} else {
			return false;
		}
	}
}