<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  UI
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

class MplayModuloSitios extends MplayModulo
{
	protected $id = 'sitios';
	protected $nombre = 'Sitios';
	protected $icono = 'fa fa-home';
	protected $descripcion = 'Módulo de mPlay para habilitar la gestión de sitios';
	
	public function __construct($options = array()){
		$this->root_path = dirname(__FILE__);
		
		parent::__construct($options);
	}
	
	function onNearestItems($p=array()){
		$lat = $p['lat'];
		$lng = $p['lng'];
		$radio = $p['radio'];
		
		if (empty($lat) || empty($lng) || !Helpers::checkExtension('ext_sitios'))
			return false;
		
		$usr_id = Usuario::isLoged() ? Usuario::getLogedId() : 0;
		$usr_obj = Usuario::isLoged() ? Usuario::getLoged() : false;

		if (empty($radio)){
			if ($usr_obj){
				$radio = (float) MplayUsuarioHelper::getRangoVision($usr_obj); // en kilometros
			} else {
				$radio = $GLOBALS['opciones']['ext_mplay']['usuario']['rango_vision'];
			}
		}
			
		if ($usr_id > 0){
			$usr_loged = Usuario::getLoged();
			$usr_loged_seguidores = new ItemsSeguidores();
			$usr_loged_seguidores_c = $usr_loged_seguidores->ListarCount("item_id='ext_usuarios.".$usr_id."'");
			
			$usrs_where = '';
			if ($usr_loged_seguidores_c > 0){
				$usr_loged_seguidores = $usr_loged_seguidores->Listar("item_id='ext_usuarios.".$usr_id."'");
				
				$usrs_where = " OR (visibilidad = '2' AND creador IN (";
				
				$ii = 0;
				foreach ($usr_loged_seguidores as $uls){
					$ii ++;
					$usrs_where .= "'-".$uls->usuario_id."'";
					
					if ($ii < $usr_loged_seguidores_c){
						$usrs_where .= ', ';
					}
				}
				
				$usrs_where .= "))";
			}
			
			$where_loc = 'estado = "1" AND params NOT LIKE "%\"tipo\":\"casa\"%" AND ((visibilidad = "1") OR (visibilidad = "0" AND creador = "-'.$usr_id.'")'.$usrs_where.')';
		} else {
			$where_loc = 'estado = "1" AND params NOT LIKE "%\"tipo\":\"casa\"%" AND (visibilidad = "1")';
		}
		
		e_include('ext_sitios/class/sitio','ext_mplay/class/mplaySitioTipo');
		
		$casa = MplayUsuarioHelper::getCasaUsr($usr_obj, $lat, $lng);
		if (!$casa instanceof Sitio)
			return false;
			
		if ($usr_obj){
			$radio_interaccion = (float) MplayUsuarioHelper::getRangoInteraccion($usr_obj); // en kilometros
		} else {
			$radio_interaccion = (float) $GLOBALS['opciones']['ext_mplay']['usuario']['rango_interaccion'];
		}
			
		$params_casa = Helpers::toArray(json_decode($casa->params));
			
		$tipo_casa_obj = MplaySitioTipo::getInstance($params_casa['tipo'], $params_casa['propiedades']);
		
		$distancia_casa = (float) $casa->get_distancia(false);
		
		if ($tipo_casa_obj->onShow()){
			if ($distancia_casa < $radio_interaccion){
				$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".$casa->titulo_formateado."<br><small>".$casa->get_distancia(true)."</small></h4><div class='mplay-popup-description'>".$casa->contenido_formateado."</div><p>".$tipo_casa_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><hr class='light'><div class='mplay-popup-options'><div class='row'>".$tipo_casa_obj->printOpcionesBtns('mapa', $casa)."</div></div></div>";
			} else {
				$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".$casa->titulo_formateado."<br><small>".$casa->get_distancia(true)."</small></h4><p>".$tipo_casa_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><div class='mplay-popup-description'>".$casa->contenido_formateado."</div><div class='mplay-popup-alert alert alert-info'>".__("Estás demasiado lejos para interactuar con tu casa.")."</div><p><small>".sprintf(__('Estás a %s de tu casa y necesitas estar a %s como mínimo para poder interactuar con ella.'), $casa->get_distancia(true), number_format($radio_interaccion * 1000, 0).' '.__('metros'))."</small></p><div class='clearfix'></div><hr class='light'><div class='mplay-popup-options'>
  <div class='row'>
    <div class='mplay-opt col-md-12'><a href='javascript:;' class='mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e' data-lat='".(float) $casa->latitud."' data-lng='".(float) $casa->longitud."' title='".__('Ir')."'><i class='fa fa-crosshairs'></i> ".__('Ir')."</a></div>
  </div>
</div></div>";
			}
		}
		
		$ancho_icono = $GLOBALS['opciones']['ext_mplay']['ui']['ancho_icono_casas'];
		$alto_icono = $GLOBALS['opciones']['ext_mplay']['ui']['alto_icono_casas'];
		if (!empty($params_casa['ancho_icono']) && !empty($params_casa['alto_icono'])){
			$ancho_icono = $params_casa['ancho_icono'];
			$alto_icono = $params_casa['alto_icono'];
		}
		if (empty($ancho_icono)){
			$ancho_icono = 64;
		}
		if (empty($alto_icono)){
			$alto_icono = 64;
		}
		$icono = $casa->getimagen_destacada($ancho_icono, $alto_icono);
		if (empty($icono) && !empty($GLOBALS['opciones']['ext_mplay']['ui']['icono_casas'])){
			$icono = Helpers::generateImageLink($GLOBALS['opciones']['ext_mplay']['ui']['icono_casas'], $ancho_icono, $alto_icono);
		}
		
		$array_json = array( array('type'=>'Feature', 'geometry'=>array('type'=>'Point', 'coordinates'=>array((float) $casa->latitud, (float) $casa->longitud)), 'properties'=>array('nombre'=>$casa->titulo_formateado, 'imagen'=>$icono, 'ancho_icono'=>$ancho_icono, 'alto_icono'=>$alto_icono, 'distancia'=>$casa->get_distancia(true), 'content'=>$popup_content, 'tipo'=>'casa', 'propiedades_html'=>$tipo_casa_obj->printPropiedadesHtml(' class="col-xs-12"'), 'propiedades'=>$tipo_casa_obj->propiedades), 'id'=>'casa_'.$casa->id) );
		
		$mua = new Sitio();
		
		$mua_chk_location = $mua->ListarCountPorLocalizacion($lat, $lng, $radio, '', 0, 0, array($where_loc));
		if ($mua_chk_location > 0){
			
			$mua_location = $mua->ListarPorLocalizacion($lat, $lng, $radio, '', 0, 0, array($where_loc));
			
			if ($usr_obj){
				$radio_interaccion = (float) MplayUsuarioHelper::getRangoInteraccion($usr_obj); // en kilometros
			} else {
				$radio_interaccion = (float) $GLOBALS['opciones']['ext_mplay']['usuario']['rango_interaccion'];
			}
			
			foreach ($mua_location as $ml){
				$params_ml = Helpers::toArray(json_decode($ml->params));
				
				$distancia_ml = (float) $ml->get_distancia(false);
				$tipo_sitio_obj = MplaySitioTipo::getInstance($params_ml['tipo'], $params_ml['propiedades']);
				
				if ($tipo_sitio_obj->onShow()){
					if ($distancia_ml < $radio_interaccion){
						$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".$ml->titulo_formateado."<br><small>".$ml->get_distancia(true)."</small></h4><div class='mplay-popup-description'>".$ml->contenido_formateado."</div><p>".$tipo_sitio_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><hr class='light'><div class='mplay-popup-options'><div class='row'>".$tipo_sitio_obj->printOpcionesBtns('mapa', $ml)."</div></div></div>";
					} else {
						$popup_content = "<div class='mplay-popup-content'><h4 class='mplay-popup-title'>".$ml->titulo_formateado."<br><small>".$ml->get_distancia(true)."</small></h4><p>".$tipo_sitio_obj->printPropiedadesHtml(' class="col-xs-12"')."</p><div class='mplay-popup-description'>".$ml->contenido_formateado."</div><div class='mplay-popup-alert alert alert-info'>".__("Estás demasiado lejos para interactuar con el sitio.")."</div><p><small>".sprintf(__('Estás a %s del sitio y necesitas estar a %s como mínimo para poder interactuar con él.'), $ml->get_distancia(true), number_format($radio_interaccion * 1000, 0).' '.__('metros'))."</small></p><div class='clearfix'></div><hr class='light'><div class='mplay-popup-options'>
  <div class='row'>
    <div class='mplay-opt col-md-12'><a href='javascript:;' class='mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e' data-lat='".(float) $ml->latitud."' data-lng='".(float) $ml->longitud."' title='".__('Ir')."'><i class='fa fa-crosshairs'></i> ".__('Ir')."</a></div>
  </div>
</div></div>";
					}
					
					$ancho_icono = $GLOBALS['opciones']['ext_mplay']['ui']['ancho_icono_sitios'];
					$alto_icono = $GLOBALS['opciones']['ext_mplay']['ui']['alto_icono_sitios'];
					if (!empty($params_ml['ancho_icono']) && !empty($params_ml['alto_icono'])){
						$ancho_icono = $params_ml['ancho_icono'];
						$alto_icono = $params_ml['alto_icono'];
					}
					if (empty($ancho_icono)){
						$ancho_icono = 64;
					}
					if (empty($alto_icono)){
						$alto_icono = 64;
					}
					$icono = $ml->getimagen_destacada($ancho_icono, $alto_icono);
					if (empty($icono) && !empty($GLOBALS['opciones']['ext_mplay']['ui']['icono_sitios'])){
						$icono = Helpers::generateImageLink($GLOBALS['opciones']['ext_mplay']['ui']['icono_sitios'], $ancho_icono, $alto_icono);
					}
					
					$tmp_array = array('type'=>'Feature', 'geometry'=>array('type'=>'Point', 'coordinates'=>array((float) $ml->latitud, (float) $ml->longitud)), 'properties'=>array('nombre'=>$ml->titulo_formateado, 'imagen'=>$icono, 'ancho_icono'=>$ancho_icono, 'alto_icono'=>$alto_icono, 'distancia'=>$ml->get_distancia(true), 'content'=>$popup_content, 'tipo'=>'sitio', 'propiedades_html'=>$tipo_sitio_obj->printPropiedadesHtml(' class="col-xs-12"'), 'propiedades'=>$tipo_sitio_obj->propiedades), 'id'=>'sitio_'.$ml->id);
					
					$array_json[] = $tmp_array;
				}
			}
			
			return $array_json;
		} else {
			return $array_json;
		}
	}
}