<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayHabilidadTipo
{
	/**
	 * @var    array  MplayHabilidadTipo instances container.
	 * @since  11.3
	 */
	protected static $instances = array();
	
	protected $name;
	protected $description;
	
	protected $todas_acciones = array();
	
	protected $acciones;
	
	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{
		$this->getTodasAcciones();
		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		} else {
			$current_options = $this->propiedades = array(
				"items"		=>	array(
										"nombre"	=>	__("Items"), 
										"tipo"		=>	"string", 
										"input"		=>	"mplayitemslist", 
										"icono"		=>	"IT", 
										"descripcion" =>	__("Items que consigue el jugador cuando aprende la habilidad"), 
										"params"	=>	array("multiple"=>"true", "models_only"=>"true"),
										"visible"	=>	false
								),
				"monedas"		=>	array(
										"nombre"	=>	__("Monedas"), 
										"tipo"		=>	"int", 
										"input"		=>	"number", 
										"icono"		=>	"MO", 
										"descripcion" =>	__("Monedas de Oro (MO) que consigue el jugador cuando aprende la habilidad"), 
										"params"	=>	array("class"=>"form-control"),
								),
				"juego"		=>	array(
										"nombre"	=>	__("Juego"), 
										"tipo"		=>	"string", 
										"input"		=>	"itemslist", 
										"icono"		=>	"TU", 
										"descripcion" =>	__("Juego que se activa cuando se aprende esta habilidad"), 
										"params"	=>	array("forzar_extension"=>"ext_mplay.1"),
										"visible"	=>	false
								),
				"ataque"	=>	array(
										"nombre"	=>	__("Ataque"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"PA",
										"descripcion"	=>	__("Puntos de Ataque (PA) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"defensa"	=>	array(
										"nombre"	=>	__("Defensa"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"PD",
										"descripcion"	=>	__("Puntos de Defensa (PD) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"vida"	=>	array(
										"nombre"	=>	__("Vida"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"PV",
										"descripcion"	=>	__("Puntos de Vida (PV) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"velocidad"	=>	array(
										"nombre"	=>	__("Velocidad"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"VM",
										"descripcion"	=>	__("Incremento de Velocidad de Movimiento (VM) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"rango_vision"	=>	array(
										"nombre"	=>	__("Rango de Visión"), 
										"tipo"		=>	"float",
										"input"		=>	"number",
										"icono"		=>	"RV",
										"descripcion"	=>	__("Incremento de Rango de Visión (RV) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"rango_interaccion"	=>	array(
										"nombre"	=>	__("Rango de Interacción"), 
										"tipo"		=>	"float",
										"input"		=>	"number",
										"icono"		=>	"RI",
										"descripcion"	=>	__("Incremento de Rango de Interacción (RV) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"carisma"	=>	array(
										"nombre"	=>	__("Carisma"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"NC",
										"descripcion"	=>	__("Incremento del Nivel de Carisma (NC) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"especial"	=>	array(
										"nombre"	=>	__("Especial"), 
										"tipo"		=>	"int",
										"input"		=>	"number",
										"icono"		=>	"PE",
										"descripcion"	=>	__("Puntos Especial (PE) que otorga la habilidad"),
										"params"	=>	array("class"=>"form-control")
										),
				"sitios"		=>	array(
										"nombre"	=>	__("Donde se puede aprender"), 
										"tipo"		=>	"string", 
										"input"		=>	"text", 
										"icono"		=>	"SI", 
										"descripcion" =>	__("Sitios donde se puede aprender la habilidad")
								)
				);
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				if (array_key_exists($kopt, $current_options)){
					$current_options[$kopt]['valor'] = $opt;
				}
			}
			
			$this->propiedades = $current_options;
		}
	}
	
	private function getTodasAcciones(){
		$this->todas_acciones = array(	
			'aprender'	=>
				array(	
					'nombre'		=>	__('Aprender'), 
					'descripcion'	=>	__('Aprender la habilidad'),
					'icon'			=>	'fa fa-flask'
				), 
			'aprendida'	=>
				array(
					'nombre'		=>	__('Aprendida'), 
					'descripcion'	=>	__('Habilidad aprendida'),
					'icon'			=>	'fa fa-flask'
				),
			'construir'	=>
				array(	
					'nombre'		=>	__('Construir'), 
					'descripcion'	=>	__('Construir la mejora'),
					'icon'			=>	'fa fa-gears'
				), 
			'construida'	=>
				array(
					'nombre'		=>	__('Construida'), 
					'descripcion'	=>	__('Mejora construida'),
					'icon'			=>	'fa fa-gears'
				)
			);
	}

	/**
	 * Returns a session storage handler object, only creating it if it doesn't already exist.
	 *
	 * @param   string  $name     The session store to instantiate
	 * @param   array   $options  Array of options
	 *
	 * @return  SesionStorage
	 *
	 * @since   11.1
	 */
	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplayHabilidadTipo' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/habilidades/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				die('Unable to load item type class: ' . $name);
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	function printOpcionesBtns($tipo, $item){
		if (empty($tipo) || !$item instanceof MplayHabilidadUsuario)
			return '';

		$acts = $this->acciones;
		
		$usr = Usuario::getLoged();
		
		$padre_item = $item->getpadre(true);
		
		if (is_object($padre_item)){
			if ($padre_item->aprendida($usr) === false && $padre_item->tipo == 'default'){
				$out = '<div class="col-xs-12">';
				$out .= '<div class="alert alert-danger mplay-alert">';
				$out .= '<i class="fa fa-exclamation-circle"></i> '.sprintf(__('Aprende <b>%s</b> para desbloquear'), SistemaIdioma::Imprimir($padre_item->nombre));
				$out .= '</div>';
				$out .= '</div>';
				
				return $out;
			} elseif ($padre_item->construida($usr) === false && $padre_item->tipo == 'sitio'){
				$out = '<div class="col-xs-12">';
				$out .= '<div class="alert alert-danger mplay-alert">';
				$out .= '<i class="fa fa-exclamation-circle"></i> '.sprintf(__('Construye <b>%s</b> para desbloquear'), SistemaIdioma::Imprimir($padre_item->nombre));
				$out .= '</div>';
				$out .= '</div>';
				
				return $out;
			}
		}
		
		if (in_array('aprender', $acts) && $item->aprendida($usr) === true && $item->tipo == 'default'){
			foreach ($acts as $kact=>$act){
				if ($act == 'aprender')
					unset($acts[$kact]);
			}
		}
		
		if (in_array('aprendida', $acts) && $item->aprendida($usr) === false && $item->tipo == 'default'){
			foreach ($acts as $kact=>$act){
				if ($act == 'aprendida')
					unset($acts[$kact]);
			}
		}
		
		if (in_array('construir', $acts) && $item->construida($usr) === true && $item->tipo == 'sitio'){
			foreach ($acts as $kact=>$act){
				if ($act == 'construir')
					unset($acts[$kact]);
			}
		}
		
		if (in_array('construida', $acts) && $item->construida($usr) === false && $item->tipo == 'sitio'){
			foreach ($acts as $kact=>$act){
				if ($act == 'construida')
					unset($acts[$kact]);
			}
		}
		
		if (!is_array($acts) || sizeof($acts) == 0){
			return '';
		}
		
		$cols = (int) (12 / count($acts));
		$class_cols = 'col-md-'.$cols;
		
		$out = '';
		
		foreach ($acts as $ac){
			$ac_arr = $this->todas_acciones[$ac];
			
			if (!is_array($ac_arr) || sizeof($ac_arr) == 0){
				continue;
			}
			
			$out .= '<div class="mplay-opt '.$class_cols.'">';
			$out .= $this->printOpcion($ac, $ac_arr, $item);
			$out .= '</div>';
		}
		
		return $out;
	}
	
	private function printOpcion($idopt=null, $options=array(), $item){
		if (empty($idopt))
			return '';
			
		$tmp_html = '';
		$propiedades = Helpers::toArray(json_decode($item->propiedades));
			
		switch ($idopt){
			case 'aprender':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-learnskill tip-e" data-skillid="'.$item->id.'" data-skill-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-valor="'.$item->coste.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].' ('.number_format($item->coste, 0, ',', '.').' MO)</a>';
				break;
			case 'aprendida':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-learnedskill tip-e disabled" data-skillid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.__('¡Habilidad aprendida!').'</a>';
				break;
			case 'construir':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-buildmejora tip-e" data-skillid="'.$item->id.'" data-skill-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-valor="'.$item->coste.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].' ('.number_format($item->coste, 0, ',', '.').' MO)</a>';
				break;
			case 'construida':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-builtmejora tip-e disabled" data-skillid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.__('¡Mejora construida!').'</a>';
				break;
			default:
				$tmp_html = '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block tip-e" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesHtml($attr=''){
		if (!is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '<div class="mplay-item-propiedades row">';	
		
		$con_false=false;
		foreach ($this->get_propiedades_parsed() as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.' id="mplay-itemprop-'.$kopt.'">';
			$tmp_html .= $this->printPropiedad($kopt, $opt);
			$tmp_html .= '</div>';
		}
		
		$tmp_html .= '</div>';
		
		return $tmp_html;
	}
	
	function get_propiedades_parsed(){
		$propiedades = $this->propiedades;
		$props_parsed = array();
		
		foreach ($propiedades as $kp=>$p){
			if ($p['visible'] !== false && !empty($p['valor']) && $p['valor'] != '0'){
				$props_parsed[$kp] = $p;
			}
		}
		
		return $props_parsed;
	}
	
	private function printPropiedad($idval=null, $options=array()){
		if (empty($idval))
			return '';
			
		$tmp_html = '';
			
		switch ($options['tipo']){
			case 'int':
			case 'float':
				$pre = '';
				if (substr($options['valor'], 0, 1) != '-'){
					$pre = '+';
				} else {
					$pre = '-';
				}
				$tmp_html .= '<p class="mplay-prop-valor">'.$pre.$options['valor'].' '.$options['nombre'].'</p>';
				break;
			case 'string':
				$tmp_html .= '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].'</p>';
				break;
			default:
				$tmp_html = '';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesForm($nombre_campo=null, $attr=''){
		if (empty($nombre_campo) || !is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '';	
		
		foreach ($this->propiedades as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.'>';
			$tmp_html .= $this->printPropiedadInput($nombre_campo.'['.$kopt.']', $opt);
			$tmp_html .= '</div>';
		}
		
		return $tmp_html;
	}
	
	private function printPropiedadInput($nombre_campo=null, $options=array()){
		if (empty($nombre_campo))
			return '';
			
		$tmp_html = $this->renderCampo($options['input'], $nombre_campo, $options['descripcion'], $options['nombre'], $options['valor'], '', '', '', $options['params']);
			
		return $tmp_html;
	}
	
	private function renderCampo($tipo='',$nombre='',$contenido='',$label='',$valor='',$formato_label='',$formato_input='', $grupo='', $params=''){
		if (empty($tipo))
			return false;
			
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $tipo);
		
		$ff_obj = new $ff;
		
		$id = Helpers::parseaTexto($nombre,true,'_');
		$descripcion = $contenido;
		
		if (!empty($grupo)){
			$nombre = $grupo."[".$nombre."]";
			//$id = $grupo.'_'.$id;
		}
		
		$ff_obj->__set('id', $id);
		$ff_obj->__set('name', $nombre);
		$ff_obj->__set('fieldname', $nombre);
		$ff_obj->__set('label', $label);
		$ff_obj->__set('description', $descripcion);
		$ff_obj->__set('valor', $valor);
		$ff_obj->__set('value', $valor);
		
		if ($tipo == 'hidden'){
			$ff_obj->__set('hidden', true);
		}
		//$params_lng = SistemaIdioma::Imprimir($this->params);
		$element = $params;
		
		$element["name"] = $nombre;
		$element["type"] = $tipo;
		$element["label"] = $label;

		$element_xml = new SimpleXMLElement('<root></root>');
		
		foreach (array_keys($element) as $key){
			
			if (is_array($element[$key])){
				if ($key == 'opciones'){
					foreach ($element[$key] as $kval=>$cval){
						$child_xml = $element_xml->addChild('option',$cval);
						$child_xml->addAttribute('value',$kval);						
					}
				} else {
					$child_xml = $element_xml->addChild($key);
					
					foreach ($element[$key] as $kval=>$cval){
						$cxml1 = $child_xml->addAttribute($kval, $cval);
					}
				}
				
			} else {
				$cxml = $element_xml->addAttribute($key, $element[$key]);
			}
		}
		
		$ff_obj->element = $element_xml;
		
		$input = $ff_obj->getInput($valor);
		
		if (empty($formato_label)){
			$formato_label = '<label for="##ID##">##LABEL##</label>
			';
		}
		if (empty($label)){
			$formato_label = '';
		}
		if (empty($formato_input)){
			$formato_input = '##INPUT##
			';
		}
		
		switch ($tipo){ 
			case 'hidden':
				$formato_label = '';
				$formato_input = "##INPUT##";
				break;
			case 'submit':
				$formato_label = '';
				break;
			case 'button':
				$formato_label = '';
				break;
		}
		if ($ff_obj->element['hide_label'] == 'true'){
			$formato_label = '';
		}
		$formato = $formato_label.$formato_input;
		$out = str_replace("##ID##", $id, $formato);
		if (!empty($label)){
			$out = str_replace("##LABEL##", $label, $out);
		}
		$out = str_replace("##INPUT##", $input, $out);
		
		return $out;
	}
	
	public static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;

		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';

		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {

			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));

				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		$file = $paths[0].'/'.strtolower($type).'.php';
		if (is_file($file)) {
			require_once $file;
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	
	function onLearn(){
		return true;
	}
	
	function onForget(){
		return true;
	}
	
	function test(){
		return true;
	}
}