<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayItemTipo
{
	/**
	 * @var    array  MplayItemTipo instances container.
	 * @since  11.3
	 */
	protected static $instances = array();
	
	protected $name;
	protected $description;
	
	protected $todas_acciones = array();
	
	protected $acciones_mapa;
	protected $acciones_inventario;
	protected $acciones_inventario_comercio;
	protected $acciones_comercio;
	protected $acciones_loot;
	protected $acciones_loot_jugador;
	
	var $tipo_slots;
	
	var $reutilizable;

	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{
		$this->getTodasAcciones();
		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		} else {
			$current_options = $this->propiedades = array("valor"=>array("nombre"=>__("Valor"), "tipo"=>"int", "input"=>"number", "icono"=>"MO", "descripcion"=>__("Valor del item en Monedas de Oro (MO)"), "params"=>array("class"=>"form-control")));
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				if (array_key_exists($kopt, $current_options)){
					$current_options[$kopt]['valor'] = $opt;
				}
			}
			
			$this->propiedades = $current_options;
		}
	}
	
	private function getTodasAcciones(){
		$this->todas_acciones = array(	
			'soltar'	=>
				array(	
					'nombre'		=>	__('Soltar'), 
					'descripcion'	=>	__('Soltar el item n la ubicación actual'),
					'icon'			=>	'fa fa-times'
				), 
			'vender'	=>
				array(
					'nombre'		=>	__('Vender'), 
					'descripcion'	=>	__('Vender el item a otro usuario'),
					'icon'			=>	'fa fa-usd'
				),
			'comprar'	=>
				array(
					'nombre'		=>	__('Comprar'), 
					'descripcion'	=>	__('Comprar el item de una tienda o usuario'),
					'icon'			=>	'fa fa-usd'
				),
			'equipar'	=>
				array(
					'nombre'		=>	__('Equipar'), 
					'descripcion'	=>	__('Equipar el item'),
					'icon'			=>	'fa fa-download'
				),
			'desequipar'	=>
				array(
					'nombre'		=>	__('Desequipar'), 
					'descripcion'	=>	__('Desequipar el item'),
					'icon'			=>	'fa fa-upload'
				),
			'ir'		=>
				array(
					'nombre'		=>	__('Ir'), 
					'descripcion'	=>	__('Ir a la posición del item en el mapa'),
					'icon'			=>	'fa fa-crosshairs'
				),
			'recoger'	=>
				array(
					'nombre'		=>	__('Recoger'), 
					'descripcion'	=>	__('Recoger el item y guardarlo en el inventario'),
					'icon'			=>	'fa fa-suitcase'
				),
			'usar'	=>
				array(
					'nombre'		=>	__('Usar'), 
					'descripcion'	=>	__('Usar el item'),
					'icon'			=>	'fa fa-check'
				),
			'leer'	=>
				array(
					'nombre'		=>	__('Leer'), 
					'descripcion'	=>	__('Leer el item'),
					'icon'			=>	'fa fa-book'
				),
			'almacenar'	=>
				array(
					'nombre'		=>	__('Almacenar'), 
					'descripcion'	=>	__('Almacenar el item'),
					'icon'			=>	'fa fa-archive'
				)
			);
	}

	/**
	 * Returns a session storage handler object, only creating it if it doesn't already exist.
	 *
	 * @param   string  $name     The session store to instantiate
	 * @param   array   $options  Array of options
	 *
	 * @return  SesionStorage
	 *
	 * @since   11.1
	 */
	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplayItemTipo' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/items/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				die('Unable to load item type class: ' . $name);
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	function printOpcionesBtns($tipo, $item){
		if (empty($tipo) || !$item instanceof MplayItem)
			return '';
			
		if ($tipo == 'mapa'){
			$acts = $this->acciones_mapa;
		} elseif ($tipo == 'inventario') {
			$acts = $this->acciones_inventario;
		} elseif ($tipo == 'comercio') {
			$acts = $this->acciones_comercio;
		} elseif ($tipo == 'inventario_comercio') {
			$acts = $this->acciones_inventario_comercio;
		} elseif ($tipo == 'loot') {
			$acts = $this->acciones_loot;
		} elseif ($tipo == 'loot_jugador') {
			$acts = $this->acciones_loot_jugador;
		}
		
		if (in_array('equipar', $acts) && $item->esta_equipado($this->tipo_slots, $this->propiedades['slot']['valor']) === true){
			foreach ($acts as $kact=>$act){
				if ($act == 'equipar')
					unset($acts[$kact]);
			}
		}
		
		if (in_array('desequipar', $acts) && $item->esta_equipado($this->tipo_slots, $this->propiedades['slot']['valor']) === false){
			foreach ($acts as $kact=>$act){
				if ($act == 'desequipar')
					unset($acts[$kact]);
			}
		}
		
		if (!is_array($acts) || sizeof($acts) == 0){
			return '';
		}
		
		$cols = (int) (12 / count($acts));
		$class_cols = 'col-md-'.$cols;
		
		$out = '';
		
		foreach ($acts as $ac){
			$ac_arr = $this->todas_acciones[$ac];
			
			if (!is_array($ac_arr) || sizeof($ac_arr) == 0){
				continue;
			}
			
			$out .= '<div class="mplay-opt '.$class_cols.'">';
			$out .= $this->printOpcion($ac, $ac_arr, $item);
			$out .= '</div>';
		}
		
		return $out;
	}
	
	private function printOpcion($idopt=null, $options=array(), $item){
		if (empty($idopt))
			return '';
			
		$tmp_html = '';
		$propiedades = Helpers::toArray(json_decode($item->propiedades));
			
		switch ($idopt){
			case 'ir':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e" data-lat="'.(float) $item->latitud.'" data-lng="'.(float) $item->longitud.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'recoger':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-getitem tip-e" data-itemid="item_'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'soltar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-dropitem tip-e" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="item_'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'usar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-useitem tip-e" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'leer':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-readitem tip-e" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'equipar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-equipitem tip-e" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'" data-item-tiposlots="'.$this->tipo_slots.'" data-item-slot="'.$propiedades['slot'].'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'desequipar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-unequipitem tip-e" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'" data-item-tiposlots="'.$this->tipo_slots.'" data-item-slot="'.$propiedades['slot'].'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'almacenar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-archiveitem tip-e" data-item-sitio="" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'vender':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-sellitem" data-item-sitio="" data-item-precio="" data-item-valor="'.(int) $propiedades['valor'].'" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].' <span class="mplay-item-sellprice"></span></a>';
				break;
			case 'comprar':
				$sitio_item = $item->getsitio(true,true);
				
				if (!$sitio_item instanceof Sitio)
					return '';
					
				$params_sitio = Helpers::toArray(json_decode($sitio_item->params));
				$params_item = Helpers::toArray(json_decode($item->propiedades));
				
				$margen_ventas = (float) $params_sitio['propiedades']['margen_ventas'];
				
				$valor_item = (float) $params_item['valor'];
				
				$precio_item = (($margen_ventas / 100) + 1) * $valor_item;
				$precio_formateado = number_format($precio_item, 0, '.', ',');
				$precio_formateado_export = number_format($precio_item, 0, '', '');

				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-buyitem" data-item-sitio="'.$item->sitio.'" data-item-precio="'.$precio_formateado_export.'" data-item-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-itemid="'.$item->id.'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].' ('.$precio_formateado.' MO)</a>';
				break;
			default:
				$tmp_html = '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block tip-e" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesHtml($attr=''){
		if (!is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '<div class="mplay-item-propiedades row">';	
		
		$con_false=false;
		foreach ($this->get_propiedades_parsed() as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.' id="mplay-itemprop-'.$kopt.'">';
			$tmp_html .= $this->printPropiedad($kopt, $opt);
			$tmp_html .= '</div>';
		}
		
		$tmp_html .= '</div>';
		
		return $tmp_html;
	}
	
	function get_propiedades_parsed(){
		$propiedades = $this->propiedades;
		$props_parsed = array();
		
		foreach ($propiedades as $kp=>$p){
			if ($p['visible'] !== false && !empty($p['valor']) && $p['valor'] != '0'){
				$props_parsed[$kp] = $p;
			}
		}
		
		return $props_parsed;
	}
	
	private function printPropiedad($idval=null, $options=array()){
		if (empty($idval))
			return '';
			
		$tmp_html = '';
		
		if ($idval == 'valor'){
			$tmp_html = '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].' MO</p>';
			
			return $tmp_html;
		}
			
		switch ($options['tipo']){
			case 'int':
			case 'float':
				$pre = '';
				if (substr($options['valor'], 0, 1) != '-'){
					$pre = '+';
				} else {
					$pre = '-';
				}
				$tmp_html .= '<p class="mplay-prop-valor">'.$pre.$options['valor'].' '.$options['nombre'].'</p>';
				break;
			case 'string':
				$tmp_html .= '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].'</p>';
				break;
			default:
				$tmp_html = '';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesForm($nombre_campo=null, $attr=''){
		if (empty($nombre_campo) || !is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '';	
		
		foreach ($this->propiedades as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.'>';
			$tmp_html .= $this->printPropiedadInput($nombre_campo.'['.$kopt.']', $opt);
			$tmp_html .= '</div>';
		}
		
		return $tmp_html;
	}
	
	private function printPropiedadInput($nombre_campo=null, $options=array()){
		if (empty($nombre_campo))
			return '';
			
		$tmp_html = $this->renderCampo($options['input'], $nombre_campo, $options['descripcion'], $options['nombre'], $options['valor'], '', '', '', $options['params']);
			
		/*switch ($options['tipo']){
			case 'int':
			case 'string':
			case 'float':
				$tmp_html = '<label>'.$options['nombre'].'</label>';
				$tmp_html .= '<input type="text" class="form-control" value="'.$options['valor'].'" name="'.$nombre_campo.'" />';
				break;
			default:
				$tmp_html = '';
				break;
		}*/
		
		return $tmp_html;
	}
	
	private function renderCampo($tipo='',$nombre='',$contenido='',$label='',$valor='',$formato_label='',$formato_input='', $grupo='', $params=''){
		if (empty($tipo))
			return false;
			
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $tipo);
		
		$ff_obj = new $ff;
		
		$id = Helpers::parseaTexto($nombre,true,'_');
		$descripcion = $contenido;
		
		if (!empty($grupo)){
			$nombre = $grupo."[".$nombre."]";
			//$id = $grupo.'_'.$id;
		}
		
		$ff_obj->__set('id', $id);
		$ff_obj->__set('name', $nombre);
		$ff_obj->__set('fieldname', $nombre);
		$ff_obj->__set('label', $label);
		$ff_obj->__set('description', $descripcion);
		$ff_obj->__set('valor', $valor);
		$ff_obj->__set('value', $valor);
		
		if ($tipo == 'hidden'){
			$ff_obj->__set('hidden', true);
		}
		//$params_lng = SistemaIdioma::Imprimir($this->params);
		$element = $params;
		
		$element["name"] = $nombre;
		$element["type"] = $tipo;
		$element["label"] = $label;

		$element_xml = new SimpleXMLElement('<root></root>');
		
		foreach (array_keys($element) as $key){
			
			if (is_array($element[$key])){
				if ($key == 'opciones'){
					foreach ($element[$key] as $kval=>$cval){
						$child_xml = $element_xml->addChild('option',$cval);
						$child_xml->addAttribute('value',$kval);						
					}
				} else {
					$child_xml = $element_xml->addChild($key);
					
					foreach ($element[$key] as $kval=>$cval){
						$cxml1 = $child_xml->addAttribute($kval, $cval);
					}
				}
				
			} else {
				$cxml = $element_xml->addAttribute($key, $element[$key]);
			}
		}
		
		$ff_obj->element = $element_xml;
		
		$input = $ff_obj->getInput($valor);
		
		if (empty($formato_label)){
			$formato_label = '<label for="##ID##">##LABEL##</label>
			';
		}
		if (empty($label)){
			$formato_label = '';
		}
		if (empty($formato_input)){
			$formato_input = '##INPUT##
			';
		}
		
		switch ($tipo){ 
			case 'hidden':
				$formato_label = '';
				$formato_input = "##INPUT##";
				break;
			case 'submit':
				$formato_label = '';
				break;
			case 'button':
				$formato_label = '';
				break;
		}
		if ($ff_obj->element['hide_label'] == 'true'){
			$formato_label = '';
		}
		$formato = $formato_label.$formato_input;
		$out = str_replace("##ID##", $id, $formato);
		if (!empty($label)){
			$out = str_replace("##LABEL##", $label, $out);
		}
		$out = str_replace("##INPUT##", $input, $out);
		
		return $out;
	}
	
	public static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;

		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';

		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {

			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));

				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		$file = $paths[0].'/'.strtolower($type).'.php';
		if (is_file($file)) {
			require_once $file;
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	
	function onShow(){
		return true;
	}
	
	function onDrop(){
		return true;
	}
	
	function onPickUp(){
		return true;
	}
	
	function onRead(){
		return true;
	}
	
	function onSell(){
		return true;
	}
	
	function onBuy(){
		return true;
	}
	
	function onUse(){
		return true;
	}
	
	function onStore(){
		return true;
	}
	
	function test(){
		return true;
	}
}