<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayMisionTipo
{
	/**
	 * @var    array  MplayItemTipo instances container.
	 * @since  11.3
	 */
	protected static $instances = array();
	
	protected $name;
	protected $description;
	
	protected $todas_acciones = array();
	
	protected $acciones_mapa;

	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{
		$this->getTodasAcciones();
		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		} else {
			$current_options = $this->propiedades = array(
				"latlng"	=>	array(
								"nombre"	=>	__("Coordenadas de la zona de activación"), 
								"tipo"		=>	"string", 
								"input"		=>	"geocomplete", 
								"icono"		=>	"CA", 
								"descripcion"	=>	__("Coordenadas de la zona de activación de la misión"),
								"params"	=>	array("class"=>"form-control", "data-input-lat"=>"#ext_misiones_detalles__propiedades__lat_", "data-input-lng"=>"#ext_misiones_detalles__propiedades__lng_", "data-atributo-detalles"=>"data-loc-custom"),
								"visible"	=>	false
							), 
				"lat"	=>	array(
								"nombre"	=>	__("Latitud de la zona de activación"), 
								"tipo"		=>	"string", 
								"input"		=>	"hidden", 
								"icono"		=>	"LTA", 
								"descripcion"	=>	__("Latitud de la zona de activación de la misión"),
								"params"	=>	array("data-loc-custom"=>"lat"),
								"visible"	=>	false
							),
				"lng"	=>	array(
								"nombre"	=>	__("Longitud de la zona de activación"), 
								"tipo"		=>	"string", 
								"input"		=>	"hidden", 
								"icono"		=>	"LNA", 
								"descripcion"	=>	__("Longitud de la zona de activación de la misión"),
								"params"	=>	array("data-loc-custom"=>"lng"),
								"visible"	=>	false
							), 
				"radio"	=>	array(
								"nombre"	=>	__("Radio de la zona de activación"), 
								"tipo"		=>	"int", 
								"input"		=>	"number", 
								"icono"		=>	"RA", 
								"descripcion"	=>	__("Radio de la zona de activación de la misión"),
								"params"	=>	array("class"=>"form-control"),
								"visible"	=>	false
							), 
				"tiempo"	=>	array(
								"nombre"	=>	__("Tiempo límite (en minutos)"), 
								"tipo"		=>	"string", 
								"input"		=>	"number", 
								"icono"		=>	"TM", 
								"descripcion"	=>	__("Tiempo límite en minutos para completar la misión"), 
								"params"	=>	array("class"=>"form-control")
							),
				"jsfunc"	=>	array(
								"nombre"	=>	__("Función JS de la misión"), 
								"tipo"		=>	"string", 
								"input"		=>	"text", 
								"icono"		=>	"JS", 
								"descripcion"	=>	__("Función JS de la misión"),
								"valor"		=>	"MPlayDefaultMission",
								"visible"	=>	false
							)
						);
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				if (array_key_exists($kopt, $current_options)){
					$current_options[$kopt]['valor'] = $opt;
				}
			}
			
			$this->propiedades = $current_options;
		}
	}
	
	public function process_start_mission($datos){
		return $datos;
	}
	
	public static function addScripts(){
		$dir = dirname(__FILE__) . '/misiones';
		$handlers = Helpers::dir_list($dir,'','.php',false);
		$handlers = $handlers[dirname(__FILE__) . '/misiones'];
		
		$names = array();
		foreach ($handlers as $handler)
		{
			$name = substr($handler, 0, strrpos($handler, '.'));
			$class = 'MplayMisionTipo' . $name;

			if (!class_exists($class))
			{
				include_once dirname(__FILE__) . '/misiones/' . $name . '.php';
			}

			if (call_user_func_array(array(trim($class), 'test'), array()))
			{
				$obj = self::getInstance($name, array('jsfunc'=>'MPlay'.ucfirst($name).'Mission'));
				$obj->addScriptMission();
			}
		}
	}
	
	function addScriptMission(){
		return $GLOBALS['AppDoc']->addScriptDeclaration('
		var trMPlayDefaultMission = [];
		
		function MPlayDefaultMission(datos){
			var lat = datos.propiedades_mision.lat;
			var lng = datos.propiedades_mision.lng;
			var m = L.latLng(lat, lng);
			if (MPlayJuegos.misiones == undefined){
				MPlayJuegos.misiones = [];
			}
			MPlayJuegos.misiones[datos.id_mision] = L.circle(m, datos.propiedades_mision.radio, { color: "blue", fillColor: "#03f", fillOpacity: 0.5, clickable: false }).addTo(MPlayMap.map);
			
			if (MPlayJuegos.timeoutMPlayDefaultMission == undefined){
				MPlayJuegos.timeoutMPlayDefaultMission = [];
			}
			
			if (datos.propiedades_mision.tiempo != undefined && datos.propiedades_mision.tiempo != ""){
				trMPlayDefaultMission[datos.id_mision] = parseInt(datos.propiedades_mision.tiempo * 60);
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				$(jq_timer).removeClass("hide").html(MideasFunciones.formatoHora(trMPlayDefaultMission[datos.id_mision]));
			}
						
			MPlayJuegos.timeoutMPlayDefaultMission[datos.id_mision] = setInterval(timerMPlayDefaultMission, 1000, datos);
		}
		function timerMPlayDefaultMission(datos){
			var areaMap = MPlayJuegos.misiones[datos.id_mision];
			var bounds = areaMap.getBounds();
			var mk = MPlayMap.marker;
			
			if (trMPlayDefaultMission[datos.id_mision] != null){
				var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
				if (!$(jq_timer).is(":visible")){
					$(jq_timer).show();
				}
				$(jq_timer).html(MideasFunciones.formatoHora(trMPlayDefaultMission[datos.id_mision]));
				if (trMPlayDefaultMission[datos.id_mision] <= 0){
					MPlayMap.map.removeLayer(MPlayJuegos.misiones[datos.id_mision]);
					MPlayJuegos.mission_expired(datos);
					clearInterval(MPlayJuegos.timeoutMPlayDefaultMission[datos.id_mision]);
					$(jq_timer).hide();
				} else {
					trMPlayDefaultMission[datos.id_mision] = trMPlayDefaultMission[datos.id_mision] - 1;
				}
			}
			if (bounds.contains(mk.getLatLng())){
				MPlayMap.map.removeLayer(MPlayJuegos.misiones[datos.id_mision]);
				if (trMPlayDefaultMission[datos.id_mision] != null){
					var jq_timer = "#mplay-ui-timer-mission-"+datos.id_mision;
					$(jq_timer).hide().html("");
				}
				MPlayJuegos.mission_completed(datos);
				clearInterval(MPlayJuegos.timeoutMPlayDefaultMission[datos.id_mision]);
			}
		}
		function MPlayDefaultMissionDel(idm){
			MPlayMap.map.removeLayer(MPlayJuegos.misiones[idm]);
			clearInterval(MPlayJuegos.timeoutMPlayDefaultMission[idm]);
		}
		');
	}
	
	private function getTodasAcciones(){
		$this->todas_acciones = array(	
			'comenzar'	=>
				array(	
					'nombre'		=>	__('Comenzar'), 
					'descripcion'	=>	__('Comenzar la misión'),
					'icon'			=>	'fa fa-play'
				), 
			'cancelar'	=>
				array(
					'nombre'		=>	__('Cancelar'), 
					'descripcion'	=>	__('Cancelar la misión'),
					'icon'			=>	'fa fa-times'
				),
			'ignorar'	=>
				array(
					'nombre'		=>	__('Ignorar'), 
					'descripcion'	=>	__('Ignorar la misión y no volver a mostrarla'),
					'icon'			=>	'fa fa-ban'
				),
			'ir'		=>
				array(
					'nombre'		=>	__('Ir'), 
					'descripcion'	=>	__('Ir a la posición de la misión en el mapa'),
					'icon'			=>	'fa fa-crosshairs'
				)
			);
	}

	/**
	 * Returns a session storage handler object, only creating it if it doesn't already exist.
	 *
	 * @param   string  $name     The session store to instantiate
	 * @param   array   $options  Array of options
	 *
	 * @return  SesionStorage
	 *
	 * @since   11.1
	 */
	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplayMisionTipo' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/misiones/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				die('Unable to load item type class: ' . $name);
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	function printOpcionesBtns($tipo, $item){
		if (empty($tipo) || !$item instanceof MisionJuegoMplay)
			return '';
			
		$acts = $this->acciones_mapa;
		
		if (in_array('comenzar', $acts) && $item->iniciada() === true){
			foreach ($acts as $kact=>$act){
				if ($act == 'comenzar')
					unset($acts[$kact]);
			}
		}
		
		if (in_array('cancelar', $acts) && $item->iniciada() === false){
			foreach ($acts as $kact=>$act){
				if ($act == 'cancelar')
					unset($acts[$kact]);
			}
		}
		
		if (!is_array($acts) || sizeof($acts) == 0){
			return '';
		}
		
		$cols = (int) (12 / count($acts));
		$class_cols = 'col-md-'.$cols;
		
		$out = '';
		
		foreach ($acts as $ac){
			$ac_arr = $this->todas_acciones[$ac];
			
			if (!is_array($ac_arr) || sizeof($ac_arr) == 0){
				continue;
			}
			
			$out .= '<div class="mplay-opt '.$class_cols.'">';
			$out .= $this->printOpcion($ac, $ac_arr, $item);
			$out .= '</div>';
		}
		
		return $out;
	}
	
	private function printOpcion($idopt=null, $options=array(), $item){
		if (empty($idopt))
			return '';
			
		$tmp_html = '';
		$propiedades = Helpers::toArray(json_decode($item->propiedades));
			
		switch ($idopt){
			case 'ir':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e" data-lat="'.(float) $item->latitud.'" data-lng="'.(float) $item->longitud.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'comenzar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-startmission tip-e" data-mision-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-misionid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'cancelar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-cancelmission tip-e" data-mision-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-misionid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'ignorar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-ignoremission tip-e" data-mision-nombre="'.SistemaIdioma::Imprimir($item->nombre).'" data-misionid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			default:
				$tmp_html = '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block tip-e" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesHtml($attr=''){
		if (!is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '<div class="mplay-item-propiedades row">';	
		
		$con_false=false;
		foreach ($this->get_propiedades_parsed() as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.' id="mplay-itemprop-'.$kopt.'">';
			$tmp_html .= $this->printPropiedad($kopt, $opt);
			$tmp_html .= '</div>';
		}
		
		$tmp_html .= '</div>';
		
		return $tmp_html;
	}
	
	function get_propiedades_parsed(){
		$propiedades = $this->propiedades;
		$props_parsed = array();
		
		foreach ($propiedades as $kp=>$p){
			if ($p['visible'] !== false && !empty($p['valor']) && $p['valor'] != '0'){
				$props_parsed[$kp] = $p;
			}
		}
		
		return $props_parsed;
	}
	
	private function printPropiedad($idval=null, $options=array()){
		if (empty($idval))
			return '';
			
		$tmp_html = '';
		
		if ($idval == 'valor'){
			$tmp_html = '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].' MO</p>';
			
			return $tmp_html;
		}
			
		switch ($options['tipo']){
			case 'int':
			case 'float':
				$pre = '';
				if (substr($options['valor'], 0, 1) != '-'){
					$pre = '+';
				} else {
					$pre = '-';
				}
				$tmp_html .= '<p class="mplay-prop-valor">'.$pre.$options['valor'].' '.$options['nombre'].'</p>';
				break;
			case 'string':
				$tmp_html .= '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].'</p>';
				break;
			default:
				$tmp_html = '';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesForm($nombre_campo=null, $attr=''){
		if (empty($nombre_campo) || !is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '';	
		
		foreach ($this->propiedades as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.'>';
			$tmp_html .= $this->printPropiedadInput($nombre_campo.'['.$kopt.']', $opt);
			$tmp_html .= '</div>';
		}
		
		return $tmp_html;
	}
	
	private function printPropiedadInput($nombre_campo=null, $options=array()){
		if (empty($nombre_campo))
			return '';
			
		$tmp_html = $this->renderCampo($options['input'], $nombre_campo, $options['descripcion'], $options['nombre'], $options['valor'], '', '', '', $options['params']);
			
		/*switch ($options['tipo']){
			case 'int':
			case 'string':
			case 'float':
				$tmp_html = '<label>'.$options['nombre'].'</label>';
				$tmp_html .= '<input type="text" class="form-control" value="'.$options['valor'].'" name="'.$nombre_campo.'" />';
				break;
			default:
				$tmp_html = '';
				break;
		}*/
		
		return $tmp_html;
	}
	
	private function renderCampo($tipo='',$nombre='',$contenido='',$label='',$valor='',$formato_label='',$formato_input='', $grupo='', $params=''){
		if (empty($tipo))
			return false;
			
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $tipo);
		
		$ff_obj = new $ff;
		
		$id = Helpers::parseaTexto($nombre,true,'_');
		$descripcion = $contenido;
		
		if (!empty($grupo)){
			$nombre = $grupo."[".$nombre."]";
			//$id = $grupo.'_'.$id;
		}
		
		$ff_obj->__set('id', $id);
		$ff_obj->__set('name', $nombre);
		$ff_obj->__set('fieldname', $nombre);
		$ff_obj->__set('label', $label);
		$ff_obj->__set('description', $descripcion);
		$ff_obj->__set('valor', $valor);
		$ff_obj->__set('value', $valor);
		
		if ($tipo == 'hidden'){
			$ff_obj->__set('hidden', true);
		}
		//$params_lng = SistemaIdioma::Imprimir($this->params);
		$element = $params;
		
		$element["name"] = $nombre;
		$element["type"] = $tipo;
		$element["label"] = $label;

		$element_xml = new SimpleXMLElement('<root></root>');
		
		foreach (array_keys($element) as $key){
			
			if (is_array($element[$key])){
				if ($key == 'opciones'){
					foreach ($element[$key] as $kval=>$cval){
						$child_xml = $element_xml->addChild('option',$cval);
						$child_xml->addAttribute('value',$kval);						
					}
				} else {
					$child_xml = $element_xml->addChild($key);
					
					foreach ($element[$key] as $kval=>$cval){
						$cxml1 = $child_xml->addAttribute($kval, $cval);
					}
				}
				
			} else {
				$cxml = $element_xml->addAttribute($key, $element[$key]);
			}
		}
		
		$ff_obj->element = $element_xml;
		
		$input = $ff_obj->getInput($valor);
		
		if (empty($formato_label)){
			$formato_label = '<label for="##ID##">##LABEL##</label>
			';
		}
		if (empty($label)){
			$formato_label = '';
		}
		if (empty($formato_input)){
			$formato_input = '##INPUT##
			';
		}
		
		switch ($tipo){ 
			case 'hidden':
				$formato_label = '';
				$formato_input = "##INPUT##";
				break;
			case 'submit':
				$formato_label = '';
				break;
			case 'button':
				$formato_label = '';
				break;
		}
		if ($ff_obj->element['hide_label'] == 'true'){
			$formato_label = '';
		}
		$formato = $formato_label.$formato_input;
		$out = str_replace("##ID##", $id, $formato);
		if (!empty($label)){
			$out = str_replace("##LABEL##", $label, $out);
		}
		$out = str_replace("##INPUT##", $input, $out);
		
		return $out;
	}
	
	public static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;

		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';

		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {

			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));

				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		$file = $paths[0].'/'.strtolower($type).'.php';
		if (is_file($file)) {
			require_once $file;
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	
	function onShow(){
		return true;
	}
	
	function onStart(){
		return true;
	}
	
	function onCancel(){
		return true;
	}
	
	function onIgnore(){
		return true;
	}
	
	function onComplete(){
		return true;
	}
	
	function test(){
		return true;
	}
}