<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  UI
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayModulo
{	
	protected $id;
	protected $nombre;
	protected $icono;
	protected $descripcion;
	protected $root_path;
	
	var $tools = array();
	var $ui_elements = array();
	
	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{	
		$conf_mod = $GLOBALS['opciones']['ext_mplay']['modulos']['modulos'][$this->id]['opciones'];
		$this->propiedades = $conf_mod;
		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				$current_options[$kopt]['valor'] = $opt;
			}
			
			$this->propiedades = $current_options;
		}
		
		$this->loadTools();
		$this->loadUIElements();
	}
	
	private function loadTools(){
		e_include('ext_mplay/class/mplayToolbarTool');
		
		$path_tools = $this->root_path . '/tools/';
		
		$dir_list = Helpers::dir_list($path_tools, '','.php');
			
		$arr_dir = $dir_list[substr($path_tools,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -4);
				
				if (!in_array($filename, $results)){
					//$results[] = 'mod_'.$this->id.':'.$filename;
					$results[] = $filename;
				}
			}
		}
		
		$this->tools = $results;
	}
	
	private function loadUIElements(){		
		$path_tools = $this->root_path . '/ui/elements/';
		
		$dir_list = Helpers::dir_list($path_tools, '','.php');
			
		$arr_dir = $dir_list[substr($path_tools,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -4);
				
				if (!in_array($filename, $results)){
					//$results[] = 'mod_'.$this->id.':'.$filename;
					$results[] = $filename;
				}
			}
		}
		
		$this->ui_elements = $results;
	}
	
	public function setNombre($str){
		if (empty($str))
			return false;
			
		$this->nombre = $str;
		
		return true;
	}
	
	public function setIcono($str){
		if (empty($str))
			return false;
			
		$this->icono = $str;
		
		return true;
	}
	
	public function setDescripcion($str){
		if (empty($str))
			return false;
			
		$this->descripcion = $str;
		
		return true;
	}
	
	public function getId(){
		return $this->id;
	}
	
	public function getNombre(){
		return $this->nombre;
	}
	
	public function getIcono(){
		return $this->icono;
	}
	
	public function getDescripcion(){
		return $this->descripcion;
	}
	
	public function getToolInstance($tool=''){
		if (empty($tool))
			return false;
			
		e_include('ext_mplay/class/mplayToolbarTool');
		
		$path_tool = $this->root_path . '/tools/' . $tool . '.php';
		$class_name_tool = 'MplayToolbarTool' . ucfirst($this->getNombre()) . ucfirst($tool);
		
		if (!class_exists($class_name_tool))
			include_once $path_tool;
			
		if (!class_exists($class_name_tool))
			return false;
			
		$inst = new $class_name_tool();
		
		return $inst;
	}
	
	public function getUIElementPath($element=''){
		if (empty($element))
			return false;
					
		$path_element = $this->root_path . '/ui/elements/' . $element . '.php';
		
		if (is_file($path_element))
			return $path_element;
		else
			return false;
	}
	
	public function printAdminForm($name, $no_fieldsets=false, $data=array(), $vista=false, $class_layout="Layout", $file_class_layout="", $tabs = false, $only_tab=''){
		if (empty($this->root_path) || !is_dir($this->root_path))
			return false;
			
		if (!empty($class_layout) && $class_layout != 'Layout' && !class_exists($class_layout) && is_file($file_class_layout))
			include($file_class_layout);
			
		c_include('forms/form','admin/layout');
		
		$xml_file = $this->root_path . '/'.$this->id.'.xml';
		
		if (!is_file($xml_file))
			return false;
			
		if (count($data) == 1 && isset($data['params']))
			$data = $data['params'];
			
		return $class_layout::renderFormXML($xml_file,$name.'['.$vista.']',$data,$no_fieldsets,array(),$tabs,$only_tab);
	}

	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplayModulo' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/modulos/' . $name . '/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				die('Unable to load module type class: ' . $name);
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	public static function getAllModules(){
		$dir = dirname(__FILE__) . '/modulos/';
		
		$arr_dir = Helpers::folder_list($dir);
				
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			
			foreach ($arr_dir as $folder){
				$clspath = $dir . $folder . '/' . $folder . '.php';
				
				if (!is_file($clspath))
					continue;
				
				$results[$folder] = $clspath;
			}
			
			return $results;
		} else {
			return false;
		}
	}
	
	function onPrintHTML(){
		$html_file = $this->root_path . '/html/' . $this->id . '.php';
		
		if (is_file($html_file)){
			ob_start();
			require $html_file;
			$contents = ob_get_contents();
			ob_end_clean();
			
			return $contents;
		} else {
			return '';
		}
	}
	
	function onGetTools(){
		$results = array();
		foreach ($this->tools as $tool){
			$results[] = 'mod_'.$this->id.':'.$tool;
		}
		return $results;
	}
	
	function onGetUIElements(){
		$results = array();
		foreach ($this->ui_elements as $element){
			$results[] = 'mod_'.$this->id.':'.$element;
		}
		return $results;
	}
	
	function onNearestItems($params=array()){
		return array();
	}
		
	function onComplete(){
		return true;
	}
	
	function test(){
		return true;
	}
}