<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayModulosManager
{
	protected static $_modulos = array();
	
	protected static $_modulos_activos = array();
	
	public static function init()
	{
		self::loadModulos();
		self::loadActiveModulos();
	}
	
	private static function loadModulos(){
		e_include("ext_mplay/class/mplayModulo");
		
		$mods = MplayModulo::getAllModules();
		
		$results = array();
		
		if (is_array($mods) && sizeof($mods) > 0){
			self::$_modulos = array();
			
			foreach ($mods as $mod=>$fld_mod){
				$check_mod = MplayModulo::getInstance($mod);
				
				if (!$check_mod instanceof MplayModulo)
					continue;
				
				if (!array_key_exists($mod, self::$_modulos)){
					self::$_modulos[$mod] = $check_mod;
				}
			}
			
			return;
		} else {
			return false;
		}
	}
	
	private static function loadActiveModulos(){
		if (empty(self::$_modulos)){
			self::loadModulos();
		}
		
		$conf = $GLOBALS['opciones']['ext_mplay']['modulos']['modulos'];
		
		foreach (self::$_modulos as $kmod=>$omod){
			if ($conf[$kmod]['activo'] == '1' && !array_key_exists($kmod, self::$_modulos_activos)){
				self::$_modulos_activos[$kmod] = $omod;
			}
		}
		
		return;
	}
	
	public static function getAllModulos(){
		return self::$_modulos;
	}
	
	public static function getActiveModulos(){
		return self::$_modulos_activos;
	}
	
	public static function executeHook($hook='', $mode='string', $params=array()){
		if (empty($hook))
			return false;
			
		if (empty(self::$_modulos_activos))
			self::init();
			
		$event_name = 'on' . ucfirst($hook);
		
		if ($mode == 'array'){
			$out = array();
		} else {
			$out = '';
		}
		
		foreach (self::$_modulos_activos as $kmod=>$mod){
			if (method_exists($mod, $event_name)){
				if ($mode == 'array'){
					$out[$kmod] = $mod->$event_name($params);
				} else {
					$out .= $mod->$event_name($params);
				}
			}
		}
		
		return $out;
	}
	
	public static function checkTool($tool=''){
		if (empty($tool))
			return false;
			
		if (substr($tool, 0, 4) != 'mod_')
			return false;
			
		$tool_exp = explode(':', substr($tool, 4));
		if (!is_array($tool_exp) || sizeof($tool_exp) != 2)
			return false;
			
		$mod = $tool_exp[0];
		$tool_id = $tool_exp[1];
		
		if (empty(self::$_modulos_activos))
			self::init();
			
		if (!array_key_exists($mod, self::$_modulos_activos))
			return false;
			
		$omod = self::$_modulos_activos[$mod];
		
		if (!in_array($tool_id, $omod->tools))
			return false;
			
		foreach ($omod->tools as $tool_mod){
			if ($tool_mod == $tool_id){
				return $omod->getToolInstance($tool_id);
			}
		}
			
		return false;
	}
	
	public static function checkUIEelement($element=''){
		if (empty($element))
			return false;
			
		if (substr($element, 0, 4) != 'mod_')
			return false;
			
		$element_exp = explode(':', substr($element, 4));
		if (!is_array($element_exp) || sizeof($element_exp) != 2)
			return false;
			
		$mod = $element_exp[0];
		$element_filename = $element_exp[1];
		
		if (empty(self::$_modulos_activos))
			self::init();
			
		if (!array_key_exists($mod, self::$_modulos_activos))
			return false;
			
		$omod = self::$_modulos_activos[$mod];
		if (!in_array($element_filename, $omod->ui_elements))
			return false;
			
		foreach ($omod->ui_elements as $element_mod){
			if ($element_mod == $element_filename){
				$omod->current_element_path = $omod->getUIElementPath($element_filename);
				
				return $omod;
			}
		}
			
		return false;
	}
}