<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplaySitioTipo
{
	/**
	 * @var    array  MplaySitioTipo instances container.
	 * @since  11.3
	 */
	protected static $instances = array();
	
	protected $name;
	protected $description;
	
	protected $todas_acciones = array();
	
	protected $acciones_mapa;

	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{
		$this->getTodasAcciones();
		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		} else {
			$current_options = $this->propiedades = array("valor"=>array("nombre"=>__("Valor"), "input"=>"number", "tipo"=>"int", "icono"=>"MO", "descripcion"=>__("Precio de venta del sitio en Monedas de Oro (MO)")));
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				if (array_key_exists($kopt, $current_options)){
					$current_options[$kopt]['valor'] = $opt;
				}
			}
			
			$this->propiedades = $current_options;
		}
	}
	
	private function getTodasAcciones(){
		$this->todas_acciones = array(	
			'comprar'	=>
				array(	
					'nombre'		=>	__('Comprar'), 
					'descripcion'	=>	__('Comprar el sitio'),
					'icon'			=>	'fa fa-shopping-cart',
					'es_propietario'=>	false
				), 
			'vender'	=>
				array(
					'nombre'		=>	__('Vender'), 
					'descripcion'	=>	__('Vender el sitio a otro usuario'),
					'icon'			=>	'fa fa-usd',
					'es_propietario'=>	true
				),
			'comerciar'	=>
				array(
					'nombre'		=>	__('Comerciar'), 
					'descripcion'	=>	__('Comerciar con el sitio'),
					'icon'			=>	'fa fa-refresh',
					'es_propietario'=>	false
				),
			'aprender'	=>
				array(
					'nombre'		=>	__('Aprender'), 
					'descripcion'	=>	__('Aprender habilidades en el sitio'),
					'icon'			=>	'fa fa-flask',
					'es_propietario'=>	false
				),
			'ir'		=>
				array(
					'nombre'		=>	__('Ir'), 
					'descripcion'	=>	__('Ir a la posición del sitio en el mapa'),
					'icon'			=>	'fa fa-crosshairs'
				),
			'almacenar'		=>
				array(
					'nombre'		=>	__('Almacenar'), 
					'descripcion'	=>	__('Almacenar o recoger items del sitio'),
					'icon'			=>	'fa fa-archive'
				),
			'mejorar'		=>
				array(
					'nombre'		=>	__('Mejorar'), 
					'descripcion'	=>	__('Mejorar y amueblar el sitio'),
					'icon'			=>	'fa fa-star'
				)
			);
	}

	/**
	 * Returns a session storage handler object, only creating it if it doesn't already exist.
	 *
	 * @param   string  $name     The session store to instantiate
	 * @param   array   $options  Array of options
	 *
	 * @return  SesionStorage
	 *
	 * @since   11.1
	 */
	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplaySitioTipo' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/sitios/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				die('Unable to load place type class: ' . $name);
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	function printOpcionesBtns($tipo, $item){
		if (empty($tipo) || !$item instanceof Sitio)
			return '';
			
		if ($tipo == 'mapa'){
			$acts = $this->acciones_mapa;
		}
		
		if (!is_array($acts) || sizeof($acts) == 0){
			return '';
		}
		
		$cuenta_acc = 0;
		
		$out = '';
		
		foreach ($acts as $ac){
			$ac_arr = $this->todas_acciones[$ac];
			
			if (!is_array($ac_arr) || sizeof($ac_arr) == 0){
				continue;
			}
			
			if (isset($ac_arr['es_propietario'])){
				if (!Usuario::isLoged())
					continue;
					
				if ($ac_arr['es_propietario'] === true){
					if (!$item->es_propietario()){
						continue;
					}
				} elseif ($ac_arr['es_propietario'] === false){
					if ($item->es_propietario()){
						continue;
					}
				}
			}
			
			$arr_tmp[$cuenta_acc] = array($ac, $ac_arr, $item);
			
			$cuenta_acc ++;
		}
		
		$cols = (int) (12 / $cuenta_acc);
		$class_cols = 'col-md-'.$cols;
		
		foreach ($arr_tmp as $atmp){
			$out .= '<div class="mplay-opt '.$class_cols.'">';
			$out .= $this->printOpcion($atmp[0], $atmp[1], $atmp[2]);
			$out .= '</div>';
		}
		
		return $out;
	}
	
	private function printOpcion($idopt=null, $options=array(), $item){
		if (empty($idopt))
			return '';
			
		$tmp_html = '';
			
		switch ($idopt){
			case 'ir':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-go tip-e" data-lat="'.(float) $item->latitud.'" data-lng="'.(float) $item->longitud.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'recoger':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-getitem tip-e" data-itemid="item_'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'soltar':
				$tmp_html .= '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-dropitem tip-e" data-item-nombre="'.$item->titulo_formateado.'" data-itemid="item_'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i></a>';
				break;
			case 'comerciar':
				$tmp_html .= '<a href="#mdl-comerciar-sitio" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-comerciar tip-e" data-sitioid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'aprender':
				$tmp_html .= '<a href="#mdl-aprender-sitio" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-aprender tip-e" data-sitioid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'almacenar':
				$tmp_html .= '<a href="#mdl-almacen-casa" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-almacenar tip-e" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			case 'mejorar':
				$tmp_html .= '<a href="#mdl-mejorar-casa" class="mplay-popup-optbtn btn btn-primary btn-block mplay-popup-optbtn-mejorar tip-e" data-sitioid="'.$item->id.'" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
			default:
				$tmp_html = '<a href="javascript:;" class="mplay-popup-optbtn btn btn-primary btn-block tip-e" title="'.$options['nombre'].'"><i class="'.$options['icon'].'"></i> '.$options['nombre'].'</a>';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesHtml($attr=''){
		if (!is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '<div class="mplay-item-propiedades row">';	
		
		foreach ($this->propiedades as $kopt=>$opt){
			if ($opt['visible'] === false){
				continue;
			}
			$tmp_html .= '<div '.$attr.' id="mplay-itemprop-'.$kopt.'">';
			$tmp_html .= $this->printPropiedad($kopt, $opt);
			$tmp_html .= '</div>';
		}
		
		$tmp_html .= '</div>';
		
		return $tmp_html;
	}
	
	private function printPropiedad($idval=null, $options=array()){
		if (empty($idval))
			return '';
			
		$tmp_html = '';
		
		if ($idval == 'valor' || $idval == 'monedas'){
			$tmp_html = '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].' MO</p>';
			
			return $tmp_html;
		}
			
		switch ($options['tipo']){
			case 'int':
			case 'float':
				$pre = '';
				if (substr($options['valor'], 0, 1) != '-'){
					$pre = '+';
				} else {
					$pre = '-';
				}
				$tmp_html .= '<p class="mplay-prop-valor">'.$pre.$options['valor'].' '.$options['nombre'].'</p>';
				break;
			case 'string':
				$tmp_html .= '<p class="mplay-prop-valor">'.$options['nombre'].': '.$options['valor'].'</p>';
				break;
			default:
				$tmp_html = '';
				break;
		}
		
		return $tmp_html;
	}
	
	function printPropiedadesForm($nombre_campo=null, $attr=''){
		if (empty($nombre_campo) || !is_array($this->propiedades) || sizeof($this->propiedades) == 0)
			return '';
		
		$tmp_html = '';	
		
		foreach ($this->propiedades as $kopt=>$opt){
			if (!empty($opt['class_div'])){
				$tmp_html .= '<div class="'.$opt['class_div'].'">';
			} else {
				$tmp_html .= '<div '.$attr.'>';
			}
			$tmp_html .= $this->printPropiedadInput($nombre_campo.'['.$kopt.']', $opt);
			$tmp_html .= '</div>';
		}
		
		return $tmp_html;
	}
	
	private function printPropiedadInput($nombre_campo=null, $options=array()){
		if (empty($nombre_campo))
			return '';
			
		$tmp_html = $this->renderCampo($options['input'], $nombre_campo, $options['descripcion'], $options['nombre'], $options['valor'], '', '', '', $options['params']);
			
		/*switch ($options['tipo']){
			case 'int':
			case 'string':
			case 'float':
				$tmp_html = '<label>'.$options['nombre'].'</label>';
				$tmp_html .= '<input type="text" class="form-control" value="'.$options['valor'].'" name="'.$nombre_campo.'" />';
				break;
			default:
				$tmp_html = '';
				break;
		}*/
		
		return $tmp_html;
	}
	
	private function renderCampo($tipo='',$nombre='',$contenido='',$label='',$valor='',$formato_label='',$formato_input='', $grupo='', $params=''){
		if (empty($tipo))
			return false;
			
		c_include('forms/formField');
		
		$ff = self::loadFieldClass('field', $tipo);
		
		$ff_obj = new $ff;
		
		$id = Helpers::parseaTexto($nombre,true,'_');
		$descripcion = $contenido;
		
		if (!empty($grupo)){
			$nombre = $grupo."[".$nombre."]";
			//$id = $grupo.'_'.$id;
		}
		
		$ff_obj->__set('id', $id);
		$ff_obj->__set('name', $nombre);
		$ff_obj->__set('fieldname', $nombre);
		$ff_obj->__set('label', $label);
		$ff_obj->__set('description', $descripcion);
		$ff_obj->__set('valor', $valor);
		$ff_obj->__set('value', $valor);
		
		if ($tipo == 'hidden'){
			$ff_obj->__set('hidden', true);
		}
		//$params_lng = SistemaIdioma::Imprimir($this->params);
		$element = $params;
		
		$element["name"] = $nombre;
		$element["type"] = $tipo;
		$element["label"] = $label;

		$element_xml = new SimpleXMLElement('<root></root>');
		
		foreach (array_keys($element) as $key){
			if (is_array($element[$key])){
				if ($key == 'opciones'){
					foreach ($element[$key] as $kval=>$cval){
						$child_xml = $element_xml->addChild('option',$cval);
						$child_xml->addAttribute('value',$kval);						
					}
				} else {
					$child_xml = $element_xml->addChild($key);
					
					foreach ($element[$key] as $kval=>$cval){
						$cxml1 = $child_xml->addAttribute($kval, $cval);
					}
				}
			} else {
				$cxml = $element_xml->addAttribute($key, $element[$key]);
			}
		}
		
		$ff_obj->element = $element_xml;
		
		$input = $ff_obj->getInput($valor);
		
		if (empty($formato_label)){
			$formato_label = '<label for="##ID##">##LABEL##</label>
			';
		}
		if (empty($label)){
			$formato_label = '';
		}
		if (empty($formato_input)){
			$formato_input = '##INPUT##
			';
		}
		
		switch ($tipo){ 
			case 'hidden':
				$formato_label = '';
				$formato_input = "##INPUT##";
				break;
			case 'submit':
				$formato_label = '';
				break;
			case 'button':
				$formato_label = '';
				break;
		}
		if ($ff_obj->element['hide_label'] == 'true'){
			$formato_label = '';
		}
		$formato = $formato_label.$formato_input;
		$out = str_replace("##ID##", $id, $formato);
		if (!empty($label)){
			$out = str_replace("##LABEL##", $label, $out);
		}
		$out = str_replace("##INPUT##", $input, $out);
		
		return $out;
	}
	
	public static function loadFieldClass($entity, $type)
	{
		$class = 'Form'.ucfirst($entity).ucfirst($type);
		if (class_exists($class)) return $class;

		// Get the field search path array.
		$paths = array();
		$paths[] = CONF_REALPATH_APP.'includes/class/forms/fields';
		$paths[] = CONF_REALPATH_APP.'extensions/ext_mplay/admin/fields';

		// If the type is complex, add the base type to the paths.
		if ($pos = strpos($type, '_')) {

			// Add the complex type prefix to the paths.
			for ($i = 0, $n = count($paths); $i < $n; $i++) {
				// Derive the new path.
				$path = $paths[$i].'/'.strtolower(substr($type, 0, $pos));

				// If the path does not exist, add it.
				if (!in_array($path, $paths)) {
					array_unshift($paths, $path);
				}
			}
			// Break off the end of the complex type.
			$type = substr($type, $pos+1);
		}
		
		// Try to find the class file.
		for ($i = 0, $n = count($paths); $i < $n; $i++) {
			$file = $paths[$i].'/'.strtolower($type).'.php';
			if (is_file($file)) {
				require_once $file;
			}
		}
		
		// Check for all if the class exists.
		return class_exists($class) ? $class : false;
	}
	
	function onShow(){
		return true;
	}
	
	function onBuy(){
		return true;
	}
	
	function onSell(){
		return true;
	}
	
	function onTrade(){
		return true;
	}
	
	function test(){
		return true;
	}
	
	// Extra
	
	public static function gethabilidades($item, $only_ids=false, $return_array=false, $tipo='loot'){
		if (!$item instanceof Sitio)
			return false;
			
		e_include("ext_mplay/class/mplayHabilidadUsuario","ext_mplay/class/mplayHabilidadTipo");
		
		if (!Usuario::isLoged())
			return false;
			
		$usr_obj = Usuario::getLoged();
			
		$params_sitio = Helpers::toArray(json_decode($item->params));
		
		if ($params_sitio['tipo'] == 'casa'){
			$tmp_obj = new MplayHabilidadUsuario();
			$tmp_obj_c = $tmp_obj->ListarCount("estado='1' AND tipo='sitio'");
			if ($tmp_obj_c > 0){
				$tmp_obj = $tmp_obj->Listar("estado='1' AND tipo='sitio'");
				$habilidades_sitio = array();
				foreach ($tmp_obj as $to){
					$habilidades_sitio[] = $to->id;
				}
			}
		} elseif ($params_sitio['tipo'] == 'formacion'){
			$propiedades_sitio = $params_sitio["propiedades"];
			$habilidades_sitio = $propiedades_sitio['habilidades'];
			
			if ($propiedades_sitio['incluir_hijas'] == '1'){
				$new_habs = array();
				foreach ($habilidades_sitio as $hab_id){
					$tmp_obj = new MplayHabilidadUsuario();
					$tmp_obj->setid($hab_id);
					if ($tmp_obj = $tmp_obj->BuscarId()){
						$new_habs[] = $hab_id;
						$child_habs = $tmp_obj->getChildren();
						if (is_array($child_habs) && sizeof($child_habs) > 0){
							foreach ($child_habs as $child){
								$new_habs[] = $child->id;
							}
						}
					}
				}
				$habilidades_sitio = $new_habs;
			}
		} else {
			return false;
		}
		
		$items_c = count($habilidades_sitio);
		
		if ($items_c > 0){
			if ($only_ids == true){
				return $habilidades_sitio;
			} else {
				$items = array();
				foreach ($habilidades_sitio as $hid){
					$hobj = new MplayHabilidadUsuario();
					$hobj->setid($hid);
					if ($hobj = $hobj->BuscarId()){
						$items[] = $hobj;
					}
				}
				if ($return_array == true){
					$params_item = $params_sitio;
					$return = array();
					foreach ($items as $it){
						if ($it->tipo == 'sitio'){
							$aprendida = $it->construida($usr_obj);
						} else {
							$aprendida = $it->aprendida($usr_obj);
						}
						$tipo_item_obj = MplayHabilidadTipo::getInstance($it->tipo, Helpers::toArray(json_decode($it->propiedades)));
						$tmp_item = array("id"=>$it->id, "nombre"=>SistemaIdioma::Imprimir($it->nombre), "descripcion"=>SistemaIdioma::Imprimir($it->descripcion), "tipo"=>$it->tipo, "oficio"=>$it->oficio, "padre"=>$it->padre, "coste"=>$it->coste, "icono"=>$it->geticono(256,256), "propiedades"=>Helpers::toArray(json_decode($it->propiedades)), 'propiedades_full'=>Helpers::toArray(json_decode($it->propiedades)), "propiedades_html"=>$tipo_item_obj->printPropiedadesHtml('class="col-xs-12"'), 'opciones'=>$tipo_item_obj->printOpcionesBtns($tipo, $it), 'aprendida'=>$aprendida);
						$return[] = $tmp_item;
					}
					return $return;
				} else {
					return $items;
				}
			}
		} else {
			return array();
		}
	}
	
	public static function getitems($item, $only_ids=false, $return_array=false, $tipo='loot'){
		if (!$item instanceof Sitio)
			return false;
			
		$params_sitio = Helpers::toArray(json_decode($item->params));
		$propiedades_sitio = $params_sitio['propiedades'];
		
		$tipo_items_sql = '';
		$tipo_items_arr = array();
		if (is_array($propiedades_sitio['tipo_items']) && sizeof($propiedades_sitio['tipo_items']) > 0){
			foreach ($propiedades_sitio['tipo_items'] as $ti){
				$tipo_items_arr[] = '"'.$ti.'"';
			}
			
			$tipo_items_sql = ' AND tipo IN ('.implode(',',$tipo_items_arr).')';
		}
		
		$where = 'estado="1" AND sitio="'.$item->id.'"'.$tipo_items_sql;
		
		$items = new MplayItem();
		$items_c = $items->ListarCount($where, 'fecha_recogido', 'DESC');
		
		if ($items_c > 0){
			$items = $items->Listar($where, 'fecha_recogido', 'DESC');
			
			if ($only_ids == true){
				$ids = array();
				foreach ($items as $it){
					$ids[] = $it->id;
				}
				return $ids;
			} else {
				if ($return_array == true){
					$params_item = Helpers::toArray(json_decode($item->params));
					$return = array();
					foreach ($items as $it){
						$cantidad = 0;
						foreach ($params_item['propiedades']['items'] as $sit){
							if ($it->id == $sit->id){
								$cantidad = $sit->cantidad;
								continue;
							}
						}
						$tipo_item_obj = MplayItemTipo::getInstance($it->tipo, Helpers::toArray(json_decode($it->propiedades)));
						$tmp_item = array('nombre'=>SistemaIdioma::Imprimir($it->nombre), 'descripcion'=>SistemaIdioma::Imprimir($it->descripcion), 'id'=>$it->id, 'model_id'=>$it->getmodel(), 'cantidad'=>$cantidad, 'tipo'=>$it->tipo, 'icono'=>$it->geticono(256,256), 'propiedades_html'=>$tipo_item_obj->printPropiedadesHtml(' class="col-xs-12"'), 'propiedades'=>$tipo_item_obj->get_propiedades_parsed(), 'propiedades_full'=>Helpers::toArray(json_decode($it->propiedades)), 'opciones'=>$tipo_item_obj->printOpcionesBtns($tipo, $it));
						$return[] = $tmp_item;
					}
					return $return;
				} else {
					return $items;
				}
			}
		} else {
			return array();
		}
	}
	
	public static function fillItems($item, $fill_empty=true){
		if (!$item instanceof Sitio || $GLOBALS['opciones']['ext_mplay']['modulos']['modulos']['inventario']['activo'] != '1')
			return false;
		
		$inv_items = (array) self::getitems($item, false, true);
		$max_items = $GLOBALS['opciones']['ext_mplay']['modulos']['modulos']['inventario']['opciones']['max_items_comercios'];
		
		if (empty($max_items))
			$max_items = "6";
			
		$parsed_items = array();
		foreach ($inv_items as $it){
			$parsed_items[] = $it;
		}
		
		if (sizeof($parsed_items) < $max_items && $fill_empty == true){
			$resto_items = $max_items - count($parsed_items);
			
			for ($i = 0; $i < $resto_items; $i ++){
				$parsed_items[] = "";
			}
		} elseif (sizeof($parsed_items) > $max_items){
			array_slice($parsed_items, 0, $max_items);
		}
		
		return $parsed_items;
	}
	
	function printInventarioForm($item, $nombre_campo, $attr){
		$inventario_items = self::fillItems($item);

		if (empty($nombre_campo) || !is_array($inventario_items))
			return '';
		
		$tmp_html = '<script>
			var tpl_inv_item = \'<img src="'.CONF_URL_APP.'image.php/uploads/imagenes/templates/bootstrap/noimage.png?width=128&amp;height=128&amp;cropratio=128:128&amp;image='.CONF_URL_APP.'uploads/imagenes/templates/bootstrap/noimage.png" class="img-responsive thumbnail" style="width: 100%;">\';
			</script>
		';

		foreach ($inventario_items as $kopt=>$opt){
			$tmp_html .= '<div '.$attr.'>';
			$tmp_html .= self::printInvItemInput($nombre_campo.'['.$kopt.']', $opt);
			$tmp_html .= '</div>';
		}

		return $tmp_html;
	}
	
	private static function printInvItemInput($nombre_campo=null, $valor){
		if (empty($nombre_campo))
			return '';
			
		$item = new MplayItem();
		if (!empty($valor["id"])){
			$item->setid($valor["id"]);
			$item = $item->BuscarId();
			
			$img_inv = $item->geticono(128,128);
			
			$tmp_html = '<div class="relative tip-e" title="'.SistemaIdioma::Imprimir($item->nombre).': '.strip_tags(SistemaIdioma::Imprimir($item->descripcion)).'"><img src="'.$img_inv.'" class="img-responsive thumbnail" style="width: 100%;" />
			<div class="btn-autohide" align="center" style="position: absolute;bottom: 5px;width: 100%;"><a class="btn btn-success btn-xs iframe-config" type="button" href="'.CONF_URL_APP_ADMIN.'?extension=ext_mplay&vista=editar_item&id='.$valor['id'].'&popup=1" title="'.__('Editar item').'"><i class="fa fa-white fa-edit"> </i></a> <a class="handle btn btn-xs btn-default" href="javascript:;"><i class="fa fa-arrows"></i></a> <input type="number" class="input-sm" id="'.Helpers::parseaTexto($nombre_campo).'" style="margin-left: 5px;width: 60px;text-align: left;" name="'.$nombre_campo.'[cantidad]" value="'.$valor["cantidad"].'" /></div></div>
			<input type="hidden" id="'.Helpers::parseaTexto($nombre_campo).'-id" name="'.$nombre_campo.'[id]" value="'.$valor["id"].'" />
			';
		} else {
			$img_inv = Helpers::getImagenDesaparecida(128,128);
			
			$tmp_html = '<img src="'.$img_inv.'" class="img-responsive thumbnail" style="width: 100%;" />';
		}

		return $tmp_html;
	}
}