<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  UI
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayToolbarTool
{	
	protected $titulo;
	protected $icono;
	protected $attr_btn;
	protected $attr_ico;
	
	var $propiedades = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public function __construct($options = array())
	{		
		if (sizeof($this->propiedades) > 0){
			$current_options = $this->propiedades;
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				$current_options[$kopt]['valor'] = $opt;
			}
			
			$this->propiedades = $current_options;
		}
	}
	
	public function setTitulo($str){
		if (empty($str))
			return false;
			
		$this->titulo = $str;
		
		return true;
	}
	
	public function setIcono($str){
		if (empty($str))
			return false;
			
		$this->icono = $str;
		
		return true;
	}
	
	public function setAttrBtn($str){
		if (empty($str))
			return false;
			
		$this->attr_btn = $str;
		
		return true;
	}
	
	public function getTitulo(){
		return $this->titulo;
	}
	
	public function getIcono(){
		return $this->icono;
	}
	
	public function getAttrBtn(){
		return $this->attr_btn;
	}
	
	private function getTodasAcciones(){
		$this->todas_acciones = array(	
			'comenzar'	=>
				array(	
					'nombre'		=>	__('Comenzar'), 
					'descripcion'	=>	__('Comenzar el juego'),
					'icon'			=>	'fa fa-play'
				), 
			'cancelar'	=>
				array(
					'nombre'		=>	__('Cancelar'), 
					'descripcion'	=>	__('Cancelar el juego'),
					'icon'			=>	'fa fa-times'
				),
			'ignorar'	=>
				array(
					'nombre'		=>	__('Ignorar'), 
					'descripcion'	=>	__('Ignorar el juego y no volver a mostrarlo'),
					'icon'			=>	'fa fa-ban'
				),
			'ir'		=>
				array(
					'nombre'		=>	__('Ir'), 
					'descripcion'	=>	__('Ir a la posición del juego en el mapa'),
					'icon'			=>	'fa fa-crosshairs'
				)
			);
	}

	/**
	 * Returns a session storage handler object, only creating it if it doesn't already exist.
	 *
	 * @param   string  $name     The session store to instantiate
	 * @param   array   $options  Array of options
	 *
	 * @return  SesionStorage
	 *
	 * @since   11.1
	 */
	public static function getInstance($name = 'none', $options = array())
	{
		$name = strtolower(Helpers::parseaTexto($name, true, '_'));

		$class = 'MplayToolbarTool' . ucfirst($name);

		if (!class_exists($class))
		{
			$path = __DIR__ . '/ui/tools/' . $name . '.php';

			if (file_exists($path))
			{
				require_once $path;
			}
			else
			{
				// No attempt to die gracefully here, as it tries to close the non-existing session
				//die('Unable to load item type class: ' . $name);
				return false;
			}
		}

		$instance = new $class($options);

		return $instance;
	}
	
	function onShow(){
		return true;
	}
	
	function onStart(){
		return true;
	}
	
	function onHover(){
		return true;
	}
	
	function onClick(){
		return true;
	}
	
	function onComplete(){
		return true;
	}
	
	function test(){
		return true;
	}
}