<?php
/**
 * @package     Mideas.mPlay
 * @subpackage  Item
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('CONF_URL_APP') or die;

abstract class MplayUI
{
	/**
	 * @var    array  MplaySitioTipo instances container.
	 * @since  11.3
	 */
	protected static $_posiciones = array();
	
	protected static $_toolbar_tools = array('geolocation', 'visibility', 'go-home', 'inventory', 'zoom', 'settings');
	
	protected static $_all_toolbar_tools = array();
	
	protected static $_all_mod_toolbar_tools = array();
	
	protected static $_all_props = array();
	
	protected static $_ui_elements = array();
	
	protected static $_ui_mod_elements = array();
	
	protected static $_ui_templates = array();
	
	protected static $_data = array();
	
	/**
	 * Constructor
	 *
	 * @param   array  $options  Optional parameters.
	 *
	 * @since   11.1
	 */
	public static function init($options = array())
	{
		if (sizeof(self::$_posiciones) > 0){
			$current_pos = self::$_posiciones;
		} else {
			$conf = $GLOBALS['opciones']['ext_mplay']['ui'];
			
			$top_left = $conf['posicion_top_left'];
			$top_right = $conf['posicion_top_right'];
			$left = $conf['posicion_left'];
			$right = $conf['posicion_right'];
			$bottom = $conf['posicion_bottom'];
			
			$current_pos = self::$_posiciones = array("top-left"=>$top_left, "top-right"=>$top_right, "left"=>$left, "right"=>$right, "bottom"=>$bottom);
		}
		
		if (is_array($options) && sizeof($options) > 0){
			foreach ($options as $kopt=>$opt){
				if (array_key_exists($kopt, $current_pos)){
					$current_options[$kopt] = $opt;
				}
			}
			
			self::$_posiciones = $current_options;
		}
		
		global $AssetBundles;
		
		if ($AssetBundles instanceof AssetsHelper){
			$AssetBundles->require_bundle('googlemaps');
			$AssetBundles->require_bundle('leaflet');
			$AssetBundles->require_bundle('geolocalizacion');
		}
		
		self::loadUIElements();
		self::loadUITemplates();
		self::loadData();
		self::loadUITools();
	}
	
	public static function printPosition($pos){
		if (empty($pos) || !isset(self::$_posiciones[$pos]))
			return false;
			
		$current_pos = self::$_posiciones[$pos];
		
		$out = '';
		
		foreach ($current_pos as $ele){
			parse_str($ele, $ele_parsed);
			
			if (!is_array($ele_parsed) || count($ele_parsed) != 3)
				continue;
				
			$out .= self::printElement($ele_parsed['tipo'], $ele_parsed['titulo'], $ele_parsed['icono']);
		}
		
		return $out;
	}
	
	public static function printTemplate($tpl){
		if (empty($tpl) || empty(self::$_ui_templates[$tpl]) || !is_file(self::$_ui_templates[$tpl]))
			return false;
			
        $contents = null;

        ob_start();
        require_once self::$_ui_templates[$tpl];
        $contents = ob_get_contents();
        ob_end_clean();
        return $contents;
	}
	
	public static function getData($key){
		if (empty($key) || !isset(self::$_data[$key]))
			return false;
			
		return self::$_data[$key];
	}
	
	private static function loadData(){
		if (!Usuario::isLoged())
			return false;
			
		e_include('ext_mplay/class/helpers/mplayUsuarioHelper', 'ext_mplay/class/mplayUsuariosActivos', 'ext_mplay/class/mplayOficioUsuario');
			
		$usr_obj = Usuario::getLoged();
		
		$lat_tmp = Helpers::getSesion()->get('mplay.usuario.geolocalizacion.latitud');
		$lng_tmp = Helpers::getSesion()->get('mplay.usuario.geolocalizacion.longitud');
		
		if (!empty($lat_tmp) && !empty($lng_tmp)){
			 $lng = $lng_tmp;
			 $lat = $lat_tmp;
		} else {
			 $lat = $GLOBALS['opciones']['ext_mplay']['usuario']['lat_inicial'];
			 $lng = $GLOBALS['opciones']['ext_mplay']['usuario']['lng_final'];
		}
		
		if (empty($lat)){
			$lat = $GLOBALS['opciones']['ext_mplay']['usuario']['lat_inicial'];
		}
		if (empty($lng)){
			$lng = $GLOBALS['opciones']['ext_mplay']['usuario']['lng_final'];
		}
		
		$mua = new MplayUsuariosActivos();

		$where_mua = "id_usuario='".$usr_obj->id."'";
		
		$mua_chk = $mua->ListarCount($where_mua);
		
		if ($mua_chk == 1){
			$mua = $mua->Listar($where_mua);
			$mua = $mua[0];
		} elseif ($mua_chk == 0){
			$usr_ip = $_SERVER['REMOTE_ADDR'];
			$usr_fecha = time();
			$geolocation = Helpers::getSesion()->get('usuario.geolocalizacion.activo');
			
			$mua->setid_usuario($usr_obj->id);
			$mua->setip_usuario($usr_ip);
			$mua->setfecha_ultimo_acceso($usr_fecha);
			$mua->setlatitud($lat);
			$mua->setlongitud($lng);
			$mua->setgeolocation($geolocation);
			
			if ($lid = $mua->Guardar()){
				$mua->setid($lid);
				$mua = $mua->BuscarId();
			}
		}
		
		self::$_data['ip'] = $mua->ip_usuario;
		self::$_data['ultimo_acceso'] = $mua->fecha_ultimo_acceso;
		self::$_data['latitud'] = $mua->latitud;
		self::$_data['longitud'] = $mua->longitud;
		self::$_data['geolocation'] = $mua->geolocation;
		
		self::$_data['casa'] = MplayUsuarioHelper::getCasaUsr($usr_obj);
		self::$_data['nivel'] = MplayUsuarioHelper::getNivelUsr($usr_obj);
		
		self::$_data['icono'] = MplayUsuarioHelper::getAvatarMap($usr_obj);
		self::$_data['icono_big'] = MplayUsuarioHelper::getAvatar($usr_obj, 400);
		self::$_data['nombre'] = $usr_obj->getnombre_completo();
		self::$_data['dinero'] = MplayUsuarioHelper::getMonedas($usr_obj);
		self::$_data['puntos'] = (int) $usr_obj->puntos;
		self::$_data['vida'] = MplayUsuarioHelper::getVida($usr_obj);
		self::$_data['max_vida'] = MplayUsuarioHelper::getMaxVida($usr_obj);
		self::$_data['velocidad'] = MplayUsuarioHelper::getVelocidad($usr_obj);
		self::$_data['rvision'] = MplayUsuarioHelper::getRangoVision($usr_obj);
		self::$_data['rinteract'] = MplayUsuarioHelper::getRangoInteraccion($usr_obj);
		self::$_data['ataque'] = MplayUsuarioHelper::getAtaque($usr_obj);
		self::$_data['defensa'] = MplayUsuarioHelper::getDefensa($usr_obj);
		self::$_data['carisma'] = MplayUsuarioHelper::getCarisma($usr_obj);
		self::$_data['especial'] = MplayUsuarioHelper::getEspecial($usr_obj);
		self::$_data['slots_armas'] = MplayUsuarioHelper::getSlotsUsr($usr_obj, 'armas', true);
		self::$_data['slots_ropa'] = MplayUsuarioHelper::getSlotsUsr($usr_obj, 'ropa', true);
		
		self::$_data['inv_empty_slot'] = Helpers::generateImageLink($GLOBALS['opciones']['ext_mplay']['general']['img_invslot_empty'], 150, 150);
		self::$_data['modulos_inventario_inv_empty_slot'] = Helpers::generateImageLink($GLOBALS['opciones']['ext_mplay']['modulos']['modulos']['inventario']['opciones']['img_invslot_empty'], 150, 150);
		self::$_data['oficios'] = new MplayOficioUsuario();
		$oficios_c = self::$_data['oficios']->ListarCount("estado='1'", "fecha", "ASC");
		if ($oficios_c > 0){
			self::$_data['oficios'] = self::$_data['oficios']->Listar("estado='1'", "fecha", "ASC");
		} else {
			self::$_data['oficios'] = array();
		}

	}
	
	public static function getDataKeys(){
		if (empty(self::$_data)){
			self::loadData();
		}
		
		$keys = array_keys(self::$_data);
		
		return $keys;
	}
	
	private static function loadUITools(){
		e_include("ext_mplay/class/mplayToolbarTool", "ext_mplay/class/mplayModulosManager");
		
		$dir = dirname(__FILE__) . '/ui/tools/';
		
		$dir_list = Helpers::dir_list($dir, '','.php');
			
		$arr_dir = $dir_list[substr($dir,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			self::$_all_toolbar_tools = array();
			
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -4);
				
				$check_tool = MplayToolbarTool::getInstance($filename);
				
				if (!$check_tool instanceof MplayToolbarTool)
					continue;
				
				if (!in_array($filename, self::$_all_toolbar_tools)){
					self::$_all_toolbar_tools[] = $filename;
				}
			}
			
			self::$_all_mod_toolbar_tools = MplayModulosManager::executeHook('getTools', 'array');
		} else {
			return false;
		}
	}
	
	private static function loadUIElements(){
		e_include("ext_mplay/class/mplayModulosManager");
		$dir = dirname(__FILE__) . '/ui/elements/';
		
		$dir_list = Helpers::dir_list($dir, '','.php');
			
		$arr_dir = $dir_list[substr($dir,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -4);
				
				self::$_ui_elements[$filename] = $dir . $file;
			}
			
			self::$_ui_mod_elements = MplayModulosManager::executeHook('getUIElements', 'array');
		} else {
			return false;
		}
	}
	
	private static function loadUITemplates(){
		$dir = dirname(__FILE__) . '/ui/templates/';
		
		$dir_list = Helpers::dir_list($dir, '','.php');
			
		$arr_dir = $dir_list[substr($dir,0,-1)];
		
		$results = array();
		
		if (is_array($arr_dir) && sizeof($arr_dir) > 0){
			
			foreach ($arr_dir as $file){
				$filename = substr($file, 0, -4);
				
				self::$_ui_templates[$filename] = $dir . $file;
			}
		} else {
			return false;
		}
	}
	
	public static function getElements(){
		if (empty(self::$_ui_elements))
			self::init();
			
		$elements['core'] = self::$_ui_elements;
		$elements['modulos'] = self::$_ui_mod_elements;
			
		return $elements;
	}
	
	public static function getAllTools(){
		if (empty(self::$_all_toolbar_tools))
			self::init();
			
		$tools['core'] = self::$_all_toolbar_tools;
		$tools['modulos'] = self::$_all_mod_toolbar_tools;
			
		return $tools;
	}
	
	public static function getAllProps(){
		if (empty(self::$_all_props))
			self::$_all_props = self::getDataKeys();
			
		$all_props = self::$_all_props;
		
		$all_props[] = 'resumen-usuario';
			
		return $all_props;
	}
	
	public static function printToolbar(){
		$conf_ui = $GLOBALS['opciones']['ext_mplay']['ui'];
		
		$tools = $conf_ui["barra_herramientas"];
		$cls_tool = $conf_ui["class_btn_herramienta"];
		$mostrar_titulo = $conf_ui["mostrar_titulo_herramienta"] == "1" ? true : false;
		$mostrar_icono = $conf_ui["mostrar_icono_herramienta"] == "1" ? true : false;
		
		$out = '<div class="mplay-ui-shortcut-icons bg-custom">
		<a class="mplay-ui-mini-icon btn-icon hidden-sm hidden-md hidden-lg toggle-mplay-toolbar" onclick="$(\'#collapse-mplay-toolbar\').toggleClass(\'visible\');" href="javascript:;"><i class="fa fa-bars"></i></a>
		
		<div id="collapse-mplay-toolbar" class="hidden-xs">
		';
		
		foreach ($tools as $tool_enc){
			parse_str($tool_enc, $tool);
			
			if (!is_array($tool) || count($tool) != 3)
				continue;
				
			$tool['class'] = $cls_tool;
			$tool['mostrar_titulo'] = $mostrar_titulo;
			$tool['mostrar_icono'] = $mostrar_icono;
			
			$out .= self::printTool($tool);
		}
		$out .= '</div></div>';
		
		return $out;
	}
	
	private static function printTool($tool){
		$inst = MplayModulosManager::checkTool($tool['tipo']);
		
		if (!$inst){
			$inst = MplayToolbarTool::getInstance($tool['tipo']);
		}
		
		$out = '';
		
		if (!$inst instanceof MplayToolbarTool)
			return false;
			
		if (!empty($tool['titulo']))
			$inst->setTitulo($tool['titulo']);
			
		if (!empty($tool['icono']))
			$inst->setIcono($tool['icono']);
			
		$main_cls = $tool['class'];
		
		if (!empty($inst->propiedades['class']['valor'])){
			$main_cls .= ' '.$inst->propiedades['class']['valor'];
		}
		
		$href = 'javascript:;';
		if (!empty($tool['href'])){
			$href = $tool['href'];
		} elseif (!empty($inst->href)){
			$href = $inst->href;
		}
		
		$attr = $inst->getAttrBtn();
		
		if (!$tool['mostrar_titulo']){
			$main_cls .= ' tip-e';
			$attr .= ' title="'.$inst->getTitulo().'"';
		}
		
		$out .= '<div class="mplay-toolbar-tool">';
		
		if ($inst->propiedades['dropdown']['valor'] == true){
			$attr .= ' data-toggle="dropdown"';
			
			$out .= '<div class="btn-group">';
		}
		
		$out .= '<a href="'.$href.'" class="'.$main_cls.'" '.$attr.'>';
		if ($tool['mostrar_icono']){
			$out .= '<i class="'.$inst->getIcono().'"></i> ';
		}
		if ($tool['mostrar_titulo']){
			$out .= '<span class="mplay-tool-title bg-custom">'.$inst->getTitulo().'</span>';
		}
		
		$out .= '</a>';
		
		if ($inst->propiedades['dropdown']['valor'] == true){
			if (!empty($inst->propiedades['dropdown-menu']['valor'])){
				$out .= $inst->propiedades['dropdown-menu']['valor'];
			}
			
			$out .= '</div>';
		}
		
		$out .= '</div>';
		
		return $out;
		
	}
	
	public static function printUserBlock(){
		$conf_ui = $GLOBALS['opciones']['ext_mplay']['ui'];
		
		$props = $conf_ui["bloque_usuario"];
		$mostrar_nombre = $conf_ui["mostrar_nombre_bloque_usuario"] == "1" ? true : false;
		$mostrar_icono = $conf_ui["mostrar_icono_bloque_usuario"] == "1" ? true : false;
		$ancho_icono = $conf_ui["ancho_icono_bloque_usuario"] != "" ? $conf_ui["ancho_icono_bloque_usuario"] : 48;
		$alto_icono = $conf_ui["alto_icono_bloque_usuario"] != "" ? $conf_ui["alto_icono_bloque_usuario"] : 48;
		$mostrar_oficio = $conf_ui["mostrar_oficio_bloque_usuario"] == "1" ? true : false;
		$mostrar_props = $conf_ui["mostrar_propiedades_bloque_usuario"] == "1" ? true : false;
		$collapse_props = $conf_ui["collapse_propiedades_bloque_usuario"] == "1" ? true : false;
		$mostrar_titulo_prop = $conf_ui["mostrar_titulo_propiedades_bloque_usuario"] == "1" ? true : false;
		$mostrar_icono_prop = $conf_ui["mostrar_icono_propiedades_bloque_usuario"] == "1" ? true : false;
		$class_props = $conf_ui["class_propiedades_bloque_usuario"];
		
		$out = '<div class="mplay-ui-user-block">';
		
		if ($mostrar_icono || $mostrar_nombre || $mostrar_oficio){
			$out .= '<div class="icon_user">';
			if ($mostrar_icono){
				$out .= '<a href="javascript:;" data-mplay-center-in-player><img src="'.MplayUI::getData('icono_big').'" width="'.$ancho_icono.'" height="'.$alto_icono.'" alt=""></a>';
			}
			if ($mostrar_nombre || $mostrar_oficio){
			$out .= '<span class="boxbg bg-custom hidden-xs">';
			
			if ($mostrar_nombre){
  				$out .= '<h3>'.MplayUI::getData('nombre').'</h3>';
  			}
			if ($mostrar_oficio){
  				$out .= '<h4><span data-mplay-user-oficio-text>'.__('Esperando...').'</span></h4>';
  			}
  			$out .= '</span>'; 
  			}
 			$out .= '<div class="clearfix"></div></div>';
		}
		
		if ($mostrar_props){
			$extra_cls_props = '';
			$out .= '<div class="pull-left" style="position:relative; z-index:3; margin-left:5px;">';
			if ($collapse_props){
				$out .= '<a href="#collapsed-more-props-usr" data-toggle="collapse" class="btn btn-info btn-mplay-toolbar pull-left tip-e hidden-xs" title="'.__('Más propiedades').'" data-placement="right" style="margin-right:10px;"><i class="fa fa-plus"></i></a>';
				$extra_cls_props = ' collapse pull-left';
			}
			
			$out .= '<div class="bg-custom hidden-xs'.$extra_cls_props.'" id="collapsed-more-props-usr">
    <div class="row">';
	
			foreach ($props as $prop_enc){
				parse_str($prop_enc, $prop);
				
				if (!is_array($prop) || count($prop) != 3)
					continue;
					
				if ($mostrar_titulo_prop){
					$prop['mostrar_titulo'] = true;
				}
				
				if ($mostrar_icono_prop){
					$prop['mostrar_icono'] = true;
				}
				
				if (!empty($class_props)){
					$prop['class'] = $class_props;
				}
				
				$out .= self::printProp($prop);
			}
	
			$out .= '</div>
			</div>
			<div class="clearfix"></div>
			</div>';
			
		}
		
		$out .= '</div>';
		
		return $out;
	}
	
	private static function printProp($prop){
		if (empty($prop))
			return false;
			
		$titulo = '';
		$icono = '';
			
		$out = '';
			
		if (!empty($prop['titulo']))
			$titulo = $prop['titulo'];
			
		if (!empty($prop['icono']))
			$icono = $prop['icono'];
			
		$main_cls = $prop['class'];
		
		$attr = $prop['attr'];
		
		if (!$prop['mostrar_titulo']){
			$main_cls .= ' tip-e';
			$attr .= ' title="'.$titulo.'"';
		}
		
		$attr_ajax = '';
		$valor_prop = '';
		switch ($prop['tipo']){
			case 'puntos':
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 0, ',', '.');
				$attr_ajax = ' data-mplay-user-level';
				break;
			case 'rvision':
			case 'rinteract':
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 2, ',', '.');
				$attr_ajax = ' data-mplay-user-' . $prop['tipo'];
				break;
			default:
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 0, ',', '.');
				$attr_ajax = ' data-mplay-user-' . $prop['tipo'];
				break;
		}
		
		$out .= '<div class="mplay-prop-valor '.$main_cls.'" '.$attr.'>';
		
		if ($prop['mostrar_icono']){
			$out .= '<i class="'.$icono.'"></i> ';
		}
		if ($prop['mostrar_titulo']){
			$out .= '<span class="mplay-prop-title">'.$titulo.'</span>';
		}
		
		$out .= '<span'.$attr_ajax.'>'.$valor_prop.'</span>';
		
		$out .= '</div>';
		
		return $out;
		
	}
	
	public static function printStatusBar(){
		$conf_ui = $GLOBALS['opciones']['ext_mplay']['ui'];
		
		$props = $conf_ui["statusbar_block"];
		$mostrar_titulo_prop = $conf_ui["mostrar_titulo_propiedades_statusbar"] == "1" ? true : false;
		$mostrar_icono_prop = $conf_ui["mostrar_icono_propiedades_statusbar"] == "1" ? true : false;
		$class_props = $conf_ui["class_propiedades_statusbar"];
		
		$out = '<div class="mplay-ui-statusbar-block">
		<ul>';

		foreach ($props as $prop_enc){
			parse_str($prop_enc, $prop);
			
			if (!is_array($prop) || count($prop) != 3)
				continue;
				
			if ($mostrar_titulo_prop){
				$prop['mostrar_titulo'] = true;
			}
			
			if ($mostrar_icono_prop){
				$prop['mostrar_icono'] = true;
			}
			
			if (!empty($class_props)){
				$prop['class'] = $class_props;
			}
			
			$out .= self::printPropStatusBar($prop);
		}
		
		$out .= '</ul></div>';
		
		return $out;
	}
	
	private static function printPropStatusBar($prop){
		if (empty($prop))
			return false;
			
		$titulo = '';
		$icono = '';
			
		$out = '';
			
		if (!empty($prop['titulo']))
			$titulo = $prop['titulo'];
			
		if (!empty($prop['icono']))
			$icono = $prop['icono'];
			
		$main_cls = $prop['class'];
		
		$attr = $prop['attr'];
		
		if (!$prop['mostrar_titulo']){
			$main_cls .= ' tip-e';
			$attr .= ' title="'.$titulo.'"';
		}
		
		$attr_ajax = '';
		$attr_icon = '';
		$valor_prop = '';
		switch ($prop['tipo']){
			case 'rvision':
			case 'rinteract':
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 2, ',', '.');
				$attr_ajax = ' data-mplay-user-' . $prop['tipo'];
				break;
			case 'dinero':
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 0, ',', '.');
				$attr_ajax = ' data-mplay-user-money';
				$main_cls .= ' value_money';
				break;
			case 'nivel':
				$valor_prop = number_format(MplayUI::getData('puntos'), 0, ',', '.');
				$attr_icon = ' data-mplay-user-level-number';
				$attr_ajax = ' data-mplay-user-level';
				$main_cls .= ' value_star';
				break;
			case 'vida':
				$cvida = number_format(MplayUI::getData('vida'), 0, ',', '.');
				$mvida = number_format(MplayUI::getData('max_vida'), 0, ',', '.');
				$pvida = number_format((100 * $cvida) / $mvida, 0, ',', '.');
				
				$valor_prop = '<div class="progress-bar" role="progressbar" aria-valuenow="'.$cvida.'" aria-valuemin="0" aria-valuemax="'.$mvida.'" style="width: '.$pvida.'%;" data-mplay-user-vida> '.$cvida.'/'.$mvida.' </div>';
				$attr_ajax = '';
				$main_cls = 'progress';
			
				break;
			default:
				$valor_prop = number_format(MplayUI::getData($prop['tipo']), 0, ',', '.');
				$attr_ajax = ' data-mplay-user-' . $prop['tipo'];
				break;
		}
		
		if (!empty($main_cls)){
			$attr_ajax .= ' class="'.$main_cls.'"';
		}
		
		$out .= '<li '.$attr.'>';
		
		if ($prop['mostrar_icono']){
			$out .= '<i class="'.$icono.'"'.$attr_icon.'></i> ';
		}
		if ($prop['mostrar_titulo']){
			$out .= '<span class="mplay-prop-title">'.$titulo.'</span>';
		}
		
		$out .= '<span'.$attr_ajax.'>'.$valor_prop.'</span>';
		
		$out .= '</li>';
		
		return $out;
		
	}
	
	private static function printElement($element, $titulo='', $icono=''){
		e_include('ext_mplay/class/mplayModulosManager');
		
		$modulo = MplayModulosManager::checkUIEelement($element);
		$inst = false;
		if (!empty($modulo->current_element_path)){
			$inst = $modulo->current_element_path;
		}
		
		if (!$inst && !empty(self::$_ui_elements[$element]) && is_file(self::$_ui_elements[$element])){
			$inst = self::$_ui_elements[$element];
		}
		
		if (!is_file($inst))
			return false;
			
        $contents = null;

        ob_start();
        require_once $inst;
        $contents = ob_get_contents();
        ob_end_clean();
        return $contents;
	}
	
}